
from vsg.rules import line_above_rule


class rule_003(line_above_rule):
    '''
    Library rule 003 checks for a blank line above the library keyword.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'library', '003', 'isLibrary')




from vsg import rule
from vsg import fix

import re


class rule_009(rule.rule):
    '''
    Process rule 009 checks the "process" keyword is lowercase on the closing of a process.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '009')
        self.solution = 'Lowercase the "process" keyword.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEndProcess and not re.match('^\s*\w+\s+process', oLine.line):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(oFile.lines[iLineNumber], 'process')




import re

from vsg import rule


class rule_003(rule.rule):
    '''
    After rule 003 checks for the "after" keyword in signal assignments in reset portion of a clock process.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'after', '003')
        self.phase = 1
        self.disable = True
        self.solution = 'Remove after from signal in the reset portion of a clock process'

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.insideResetProcess and oLine.hasAfterKeyword:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            sNewLine = re.sub('\s*after.*;', ';', oLine.line, 1, flags=re.IGNORECASE)
            oLine.update_line(sNewLine)
            oLine.hasAfterKeyword = False




from vsg import rule
from vsg import check


class rule_004(rule.rule):
    '''
    Concurrent rule 004 checks there is at least a single space before the assignment.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'concurrent'
        self.identifier = '004'
        self.solution = 'Add a single space before the <=.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isConcurrentBegin:
            check.is_single_space_before_character(self, '<=', oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(oLine.line.replace('<=', ' <='))




from vsg.rules import prefix_rule
from vsg import utils


class rule_020(prefix_rule):
    '''
    Generic rule 020 checks for prefixes in generic names.
    '''

    def __init__(self):
        prefix_rule.__init__(self, 'generic', '020', 'isGenericDeclaration')
        self.prefixes = ['G_']
        self.solution = 'Generic'

    def _extract(self, oLine):
        return utils.extract_generics(oLine)




from vsg.rules.package import uppercase_package_name_rule


class rule_010(uppercase_package_name_rule):
    '''
    Package rule 010 checks the package name is upper case in the package declaration.
    '''

    def __init__(self):
        uppercase_package_name_rule.__init__(self, 'package', '010', 'isPackageKeyword')




from vsg import rule
from vsg import fix

import re


class rule_031(rule.rule):
    '''
    Instantiation rule 031 checks the component keyword is lowercase if it is used.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '031'
        self.solution = 'Lowercase the component keyword'
        self.phase = 6
        self.disable = True

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationDeclaration and not oLine.isDirectInstantiationDeclaration:
            if re.match('^\s*\w+\s*:\s*component', oLine.line, flags=re.IGNORECASE):
                if not re.match('^\s*\w+\s*:\s*component', oLine.line):
                    self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(oFile.lines[iLineNumber], 'component')




from vsg import rule
from vsg import check
from vsg import fix


class rule_030(rule.rule):
    '''
    If rule 030 checks for a blank line after the "end if" keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'if', '030')
        self.phase = 3
        self.solution = 'Add a blank line after the "end if"'

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isLastEndIf:
            check.is_blank_line_after(self, oFile, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            fix.insert_blank_line_below(self, oFile, iLineNumber)



'''
This module contains functions for rules to perform their checks.
'''

import re


def is_single_space_after_character(self, sCharacter, oLine, iLineNumber):
    '''
    Checks if a single space exists after a series of characters.
    NOTE:  The characters will match partial words.

    Parameters:

      self: (rule object)

      sCharacter: (string)

      oLine: (line object)

      iLineNumber: (integer)
    '''
    if not sCharacter.lower() in oLine.lineLower:
        return
    if not re.match('^.*' + sCharacter.lower() + '\s*--', oLine.lineNoComment):
        if re.match('^.*' + sCharacter.lower() + '$', oLine.lineNoComment):
            return
        if not re.match('^.*' + sCharacter.lower() + '\s\S', oLine.lineLower):
            self.add_violation(iLineNumber)




from vsg import rule
from vsg import fix
from vsg import check


class remove_blank_lines_above_rule(rule.rule):
    '''
    Component rule 016 checks for a blank line above the "end component" keywords.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None, sUnless=None):
        rule.rule.__init__(self, name, identifier)
        self.solution = None
        self.phase = 3
        self.sTrigger = sTrigger
        self.sUnless = sUnless

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.sTrigger]:
            check.is_no_blank_line_before(self, oFile, iLineNumber, self.sUnless)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            fix.remove_blank_lines_above(self, oFile, iLineNumber, self.sUnless)




from vsg.rules import single_space_after_rule


class rule_004(single_space_after_rule):
    '''
    Case rule 004 checks for a single space after the "when" keyword.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'case', '004', 'isCaseWhenKeyword', 'when')
        self.solution = 'Ensure a single space exists after the "when" keyword.'




from vsg import rule

import re
import copy


class rule_013(rule.rule):
    '''
    Port rule 013 checks for multiple ports declared on single line.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'port', '013')
        self.solution = 'Place multiple ports on their own lines.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPortDeclaration and re.match('^.*,.*:', oLine.lineNoComment):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            oLine = oFile.lines[iLineNumber]
            iNumberOfPorts = oLine.line.split(':')[0].count(',') + 1
            # Replicate ports
            for iIndex in range(1, iNumberOfPorts):
                oFile.lines.insert(iLineNumber, copy.deepcopy(oLine))
            # Split ports
            for iIndex in range(0, iNumberOfPorts):
                oLine = oFile.lines[iLineNumber + iIndex]
                lLine = oLine.line.split(':')
                lPorts = lLine[0].split(',')
                oLine.update_line(lPorts[iIndex] + ' :' + lLine[1])




from vsg.rules import case_rule
from vsg import utils


class rule_008(case_rule):
    '''Component rule 008 checks the component name has proper case in the component declaration line.'''

    def __init__(self):
        case_rule.__init__(self, 'component', '008', 'isComponentDeclaration')
        self.case = 'upper'
        self.solution = 'Change component name to ' + self.case + 'case'

    def _extract(self, oLine):
        return utils.extract_component_identifier(oLine)




from vsg.rules import keyword_alignment_rule


class rule_002(keyword_alignment_rule):
    '''
    After rule 002 ensures the alignment of the after keyword in clock processes
    '''

    def __init__(self):
        keyword_alignment_rule.__init__(self, 'after', '002')
        self.solution = 'Inconsistent alignment of after keywords.'
        self.sKeyword = 'after'
        self.sStartGroupTrigger = 'insideClockProcess'
        self.sEndGroupTrigger = 'isEndProcess'
        self.sLineTrigger = 'hasAfterKeyword'
        self.disable = True




from vsg import rule
from vsg import line


class rule_014(rule.rule):
    '''
    Instantiation rule 014 checks the closing ) for the generic map is on it's own line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '014'
        self.solution = 'Place closing ) on it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationGenericEnd and (oLine.isInstantiationGenericAssignment or oLine.isInstantiationGenericKeyword):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            _remove_closing_parenthesis(oFile, iLineNumber)
            _add_closing_parenthesis_on_new_line(oFile, iLineNumber)


def _remove_closing_parenthesis(oFile, iLineNumber):
    oLine = oFile.lines[iLineNumber]
    sLine = oLine.lineNoComment
    iIndex = sLine.rfind(')')
    sLine = oLine.line
    sLine = sLine[:iIndex] + sLine[iIndex + 1:]
    oLine.update_line(sLine)
    oFile.lines[iLineNumber].isInstantiationGenericEnd = False


def _add_closing_parenthesis_on_new_line(oFile, iLineNumber):
    oFile.lines.insert(iLineNumber + 1, line.line('  )'))
    oFile.lines[iLineNumber + 1].isInstantiationGenericAssignement = False
    oFile.lines[iLineNumber + 1].isInstantiationGenericEnd = True
    oFile.lines[iLineNumber + 1].insideInstantiation = True
    oFile.lines[iLineNumber + 1].indentLevel = oFile.lines[iLineNumber].indentLevel - 1





from vsg.rules import indent_rule


class rule_003(indent_rule):
    '''
    Procedure rule 003 checks the indent of the **begin** keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'procedure', '003', 'isProcedureEnd', 'isProcedureParameter')




from vsg import rule
from vsg import fix

import re


class rule_002(rule.rule):
    '''
    Process rule 002 checks there is a single space between the process
    keyword and the (.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'process'
        self.identifier = '002'
        self.solution = 'Ensure a single space exists between the "process" \
                          keyword and the (.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isProcessKeyword and re.match('^\s*.*process\s*\(', oLine.lineLower) and not re.match('^\s*.*process\s\(', oLine.lineLower):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'process')




from vsg import rule
from vsg import fix

import re


class rule_003(rule.rule):
    '''
    Function rule 003 checks there is a single space between the function name and the (.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'function'
        self.identifier = '003'
        self.solution = 'Ensure a single space exists between the function name and the (.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isFunctionKeyword and function_has_parameters(oLine.line):
            if not re.match('^\s*function\s+\w+\s\(', oLine.lineLower) and \
               not re.match('^\s*impure\s+function\s+\w+\s\(', oLine.lineLower) and \
               not re.match('^\s*pure\s+function\s+\w+\s\(', oLine.lineLower):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_before_word(self, oFile.lines[iLineNumber], '\(')


def function_has_parameters(sString):
    if re.match('^\s*function\s+\w+\s*\(', sString, re.IGNORECASE) or \
       re.match('^\s*[impure|pure]\s+function\s+\w+\s*\(', sString, re.IGNORECASE):
        return True
    else:
        return False




from vsg.rules import indent_rule


class rule_005(indent_rule):
    '''
    Type rule 005 checks for the proper indentation of multiline types.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'type', '005', 'insideTypeEnumerated', 'isTypeEnumeratedKeyword')
        self.solution = 'Ensure proper indentation.'




from vsg.rules import suffix_rule
from vsg import utils


class rule_025(suffix_rule):
    '''
    Port rule 025 checks for suffixes in port names.
    '''

    def __init__(self):
        suffix_rule.__init__(self, 'port', '025', 'isPortDeclaration')
        self.suffixes = ['_I', '_O', '_IO']
        self.solution = 'Port'

    def _extract(self, oLine):
        return utils.extract_port_name(oLine)




from vsg import rule
from vsg import fix


class rule_026(rule.rule):
    '''
    Process rule 026 checks for blank lines between the end of the sensitivity list and process declarative lines.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '026')
        self.solution = 'Ensure a single blank line between the end of the sensitivity list and the next non-blank line.'
        self.phase = 3

    def analyze(self, oFile):
        fCheckForBlanks = False
        fSkipProcess = False
        iBlankCount = 0
        iFailingLineNumber = 0
        for iLineNumber, oLine in enumerate(oFile.lines):
            if not self._is_vsg_off(oLine):
                if oLine.insideProcess:
                    if oLine.isProcessBegin and oLine.isSensitivityListEnd:
                        fSkipProcess = True
                    if oLine.isSensitivityListEnd and oFile.lines[iLineNumber + 1].isProcessBegin:
                        fSkipProcess = True
                    if fSkipProcess:
                        if oLine.isEndProcess:
                            fSkipProcess = False
                        continue  # pragma: no cover
                    if fCheckForBlanks:
                        if oLine.isBlank:
                            iBlankCount += 1
                        else:
                            if not iBlankCount == 1 and not oLine.isProcessBegin:
                                self.add_violation(iFailingLineNumber)
                            fSkipProcess = True
                            fCheckForBlanks = False
                            iBlankCount = 0
                    if oLine.isSensitivityListEnd:
                        fCheckForBlanks = True
                        iFailingLineNumber = iLineNumber

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            fix.insert_blank_line_below(self, oFile, iLineNumber)
            oFile.lines[iLineNumber + 1].insideProcess = True




from vsg import rule
from vsg import fix

import re


class rule_006(rule.rule):
    '''
    Package rule 006 checks for the "end" and "package" keyword are lower case.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'package'
        self.identifier = '006'
        self.solution = 'Ensure "end" and "package" keywords are lower case.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPackageEnd and re.match('^\s*end\s+package', oLine.lineLower):
            if not re.match('^\s*end\s+package', oLine.line):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(oFile.lines[iLineNumber], 'end')
            fix.lower_case(oFile.lines[iLineNumber], 'package')



'''
This module contains functions for rules to perform their checks.
'''

from vsg.check import are_there_spaces_after

def is_single_space_after(self, sString, oLine, iLineNumber):
    '''
    Checks if a single space is after the string given.
    The string is considered a whole word.
    Allowances are made for end of line and semicolons.

    Parameters:

      self: (rule object)

      sString: (string)

      oLine: (line object)

      iLineNumber: (integer)
    '''

    are_there_spaces_after(self, sString, oLine, iLineNumber, 1)




from vsg.rules import single_space_before_character_rule


class rule_014(single_space_before_character_rule):
    '''
    Generic rule 014 checks for at least a single space before the :.
    '''

    def __init__(self):
        single_space_before_character_rule.__init__(self, 'generic', '014', 'isGenericDeclaration', ':')
        self.solution = 'Add a space before the :.'




def has_package_name(oLine):
    '''
    Returns boolean if package name is found in line.

    Parameters:

      oLine: (line object)

    Returns: (boolean)

      True if package name exists in line.
      False if package name does not exist in line.
    '''
    lLine = oLine.lineNoComment.lower().split()
    for iIndex, sWord in enumerate(lLine):
        if sWord == 'package':
            if not lLine[iIndex + 1] == '--':
                return True
    return False




from vsg import rule

import re


class rule_029(rule.rule):
    '''
    Process rule 029 checks for "rising_edge" and "falling_edge" in processes.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'process'
        self.identifier = '029'
        self.solution = 'Use \'event for clocks.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.insideProcess and re.match('^.*ing_edge\s*\(', oLine.lineLower):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'rising_edge\s*\(\s*(\w+)\s*\)', r'\1"event and \1 = "1"', oLine.line, re.IGNORECASE).replace('"', '\''))
            oLine.update_line(re.sub(r'falling_edge\s*\(\s*(\w+)\s*\)', r'\1"event and \1 = "0"', oLine.line, re.IGNORECASE).replace('"', '\''))



from vsg.rules import prefix_rule
from vsg import utils


class rule_004(prefix_rule):
    '''
    Subtype rule 004 checks for prefixes in user defined subtype identifiers.
    '''

    def __init__(self):
        prefix_rule.__init__(self, 'subtype', '004', 'isSubtypeKeyword')
        self.prefixes = ['st_']
        self.solution = 'Subtype'

    def _extract(self, oLine):
        return utils.extract_type_identifier(oLine)




from vsg import rule
from vsg import fix
from vsg import check


class uppercase_package_name_rule(rule.rule):
    '''
    Package rule 008 checks the package name is upper case on the closing "end package" line.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None):
        rule.rule.__init__(self, name, identifier)
        self.solution = 'Uppercase package name.'
        self.phase = 6
        self.sTrigger = sTrigger

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.sTrigger] and check.has_package_name(oLine):
            check.is_uppercase(self, check.get_package_name(oLine), iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.upper_case(oFile.lines[iLineNumber], check.get_package_name(oFile.lines[iLineNumber]))




from vsg.rules import prefix_rule
from vsg import utils


class rule_011(prefix_rule):
    '''
    Port rule 011 checks for prefixes in port names.
    '''

    def __init__(self):
        prefix_rule.__init__(self, 'port', '011', 'isPortDeclaration')
        self.prefixes = ['I_', 'O_', 'IO_']
        self.solution = 'Port'

    def _extract(self, oLine):
        return utils.extract_port_name(oLine)




from vsg import rule

import re


class rule_018(rule.rule):
    '''
    Process rule 018 checks the "end process" has a label.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '018')
        self.solution = 'Add a label for the "end process".'
        self.phase = 1

    def _pre_analyze(self):
        self.dFix['processLabel'] = {}
        self.labelStack = ''
        self.iProcStartLine = 0
        self.iProcLabelLine = 0

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isProcessKeyword:
            self.iProcStartLine = iLineNumber
            oMatch = re.match('^\s*(\S+)\s*:\s*process', oLine.lineLower)
            if oMatch:
                self.iProcLabelLine = iLineNumber
                self.labelStack = oMatch.group(1)
        if oLine.isEndProcess:
            if not re.match('^\s*\S+\s+\S+\s+\S+', oLine.line):
                self.add_violation(iLineNumber)
            if self.labelStack and self.iProcStartLine == self.iProcLabelLine:
                self.dFix['processLabel'][iLineNumber] = self.labelStack
            self.labelStack = ''

    def _fix_violations(self, oFile):
        for iLineNumber, oLine in enumerate(oFile.lines):
            if oLine.isEndProcess and not re.match('^\s*\S+\s+\S+\s+\S+', oLine.line):
                if iLineNumber in self.dFix['processLabel']:
                    oMatch = re.search(r'^(\s*)\S+\s+\S+', oLine.line)
                    oLine.line = oMatch.group(1) + 'end process ' + self.dFix['processLabel'][iLineNumber] + ';'




from vsg import rule
from vsg import check
from vsg import fix

import re


class rule_009(rule.rule):
    '''
    If rule 009 checks the alignment of multiline boolean expressions.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'if'
        self.identifier = '009'
        self.solution = 'Align with space after ( in first line of boolean expression.'
        self.phase = 5

    def analyze(self, oFile):
        fCheckAlignment = False
        for iLineNumber, oLine in enumerate(oFile.lines):
            if self._is_vsg_off(oLine):
                continue
            if oLine.isIfKeyword and oLine.isThenKeyword:
                continue
            if oLine.isElseIfKeyword and oLine.isThenKeyword:
                continue
            if oLine.isIfKeyword:
                if re.match('^\s+if\s\(', oLine.lineLower):
                    iAlignmentColumn = oLine.lineLower.find('(')
                    fCheckAlignment = True
                continue  # pragma: no cover
            if oLine.isElseIfKeyword:
                if re.match('^\s+elsif\s\(', oLine.lineLower):
                    iAlignmentColumn = oLine.lineLower.find('(')
                    fCheckAlignment = True
                continue  # pragma: no cover
            if oLine.insideIf and fCheckAlignment:
                check.multiline_alignment(self, iAlignmentColumn + 1, oLine, iLineNumber)
            if oLine.isThenKeyword:
                fCheckAlignment = False

    def _fix_violations(self, oFile):
        for iLineNumber in self.dFix['violations']:
            fix.multiline_alignment(self, oFile, iLineNumber)




from vsg.rules import remove_blank_lines_above_rule


class rule_016(remove_blank_lines_above_rule):
    '''
    Entity rule 016 checks for a blank line above the "end entity" keywords.
    '''

    def __init__(self):
        remove_blank_lines_above_rule.__init__(self, 'entity', '016', 'isEndEntityDeclaration')
        self.solution = 'Remove blank line(s) above "end entity" keywords.'




from vsg.rules import case_rule
from vsg import utils


class rule_015(case_rule):
    '''
    Entity rule 015 checks the is keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'case', '015', 'isCaseIsKeyword')
        self.solution = 'Change "is" keyword to '

    def _extract(self, oLine):
        return utils.extract_word(oLine, 'is')




from vsg import rule
from vsg import check
from vsg import fix


class rule_011(rule.rule):
    '''
    Case rule 011 ensures the alignment of multiline "when" statements.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'case'
        self.identifier = '011'
        self.solution = 'Align with space after the "when" keyword.'
        self.phase = 5
        # Variables required for analysis
        self.iAlignmentColumn = 0

    def _pre_analyze(self):
        self.iAlignmentColumn = 0

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isCaseWhenKeyword:
            if not oLine.isCaseWhenEnd:
                self.iAlignmentColumn = (oLine.indentLevel * self.indentSize) + len('when ')
        elif oLine.insideCaseWhen:
            check.multiline_alignment(self, self.iAlignmentColumn, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.dFix['violations']:
            fix.multiline_alignment(self, oFile, iLineNumber)




def is_blank_line_after(self, oFile, iLineNumber):
    '''
    Adds a violation if the line after iLineNumber is not blank.
    This is typically used to compress lines together.

    Parameters

      self: (rule object)

      oFile: (vhdlFile object)

      iLineNumber: (integer)
    '''
    if not oFile.lines[iLineNumber + 1].isBlank:
        self.add_violation(iLineNumber)




from vsg.rules import indent_rule


class rule_015(indent_rule):
    '''
    Port rule 015 checks the indentation of closing parenthesis for port maps.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'port', '015', 'isEndPortMap', 'isPortDeclaration')
        self.solution = 'Ensure proper indentation.'




def has_prefix(self, prefixes, words, iLineNumber):
    '''
    Checks if words in the list have allowed prefixes.

    Parameters:

      self: (rule object)

      prefixes: (list of allowed prefixes)

      words: (list of words to check)

      iLineNumber: (integer)
    '''
    for word in words:
        for prefix in prefixes:
            if word.lower().startswith(prefix.lower()):
                break
        else:
            self.add_violation(iLineNumber)



'''
This module contains functions for rules to perform their checks.
'''

import re


def is_single_space_before_character(self, sCharacter, oLine, iLineNumber):
    '''
    Checks if a single space exists before a series of characters.
    NOTE:  The characters will match partial words.

    Parameters:

      self: (rule object)

      sCharacter: (string)

      oLine: (line object)

      iLineNumber: (integer)
    '''
    iIndex = oLine.line.find(sCharacter) + len(sCharacter)
    if not re.match('^.*\s' + sCharacter.lower(), oLine.lineNoComment[:iIndex]):
        self.add_violation(iLineNumber)




from vsg import fix
from vsg import check
from vsg import rule


class line_above_rule(rule.rule):
    '''
    Checks for blank lines above a line and will insert a blank line if one does not exist.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    sTrigger : string
       The line attribute the rule applies to.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None, sUnless=None):
        rule.rule.__init__(self, name, identifier)
        self.solution = 'Insert blank line above.'
        self.phase = 3
        # The user updates the attributes below
        self.condition = sTrigger
        self.sUnless = sUnless

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.condition]:
            check.is_blank_line_before(self, oFile, iLineNumber, self.sUnless)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            fix.insert_blank_line_above(self, oFile, iLineNumber)




from vsg.rules import remove_spaces_before_character_rule


class rule_003(remove_spaces_before_character_rule):
    '''
    Whitespace rule 003 checks for spaces before semicolons
    '''

    def __init__(self):
        remove_spaces_before_character_rule.__init__(self, 'whitespace', '003', ';')
        self.solution = 'Remove spaces before semicolons.'




from vsg import rule
from vsg import fix

import re


class rule_015(rule.rule):
    '''If rule 015 checks there is a single space between the "end" and "if" keywords.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'if'
        self.identifier = '015'
        self.solution = 'Ensure only a single space exists between the "end" and "if" keywords.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEndIfKeyword:
            if re.match('^\s*end\s+if', oLine.line, re.IGNORECASE):
                if not re.match('^\s*end\sif', oLine.line, re.IGNORECASE):
                    self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'end')



'''
This module contains functions for rules to perform their checks.
'''

import re


def is_single_space_before(self, sString, oLine, iLineNumber):
    '''
    Checks if a single space exists before the string given.
    The string is considered a whole word.
    Parameters:
      self: (rule object)
      sString: (string)
      oLine: (line object)
      iLineNumber: (integer)
    '''
    if not sString.lower() in oLine.lineLower:
        return
    if not re.match('^.*\S\s' + sString + '\s', oLine.lineNoComment.lower()) and \
       not re.match('^.*\S\s' + sString + '$', oLine.lineNoComment.lower()) and \
       not re.match('^.*\S\s' + sString + '\'', oLine.lineNoComment.lower()):
        self.add_violation(iLineNumber)




from vsg import rule
from vsg import utils

import re


class rule_009(rule.rule):
    '''
    Type rule 009 checks for enumerated types after the open parenthesis on a multi-line type declaration.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'type', '009')
        self.solution = 'Move enumerated type to it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isTypeEnumeratedKeyword and not oLine.isTypeEnumeratedEnd:
            if re.match('^.*\sis\s*\(\s*\w', oLine.lineLower):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            utils.split_line_after_word(oFile, iLineNumber, '(')
            oLine = oFile.lines[iLineNumber + 1]
            oLine.isTypeKeyword = False
            oLine.isTypeEnumeratedKeyword = False
            oLine.indentLevel += 1




from vsg import rule
from vsg import utils
import re


class rule_012(rule.rule):
    '''
    Case rule 012 ensures code does not exist after the => operator.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'case', '012')
        self.solution = 'Move code after the => operator to it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isCaseWhenEnd and re.match('^.*=>\s*\w', oLine.line):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            utils.split_line_after_word(oFile, iLineNumber, '=>')
            oFile.lines[iLineNumber + 1].isCaseWhenEnd = False
            oFile.lines[iLineNumber + 1].insideCaseWhen = False
            oFile.lines[iLineNumber + 1].indentLevel += 1
            utils.reclassify_line(oFile, iLineNumber)




def has_suffix(self, suffixes, words, iLineNumber):
    '''
    Checks if words in the list have allowed suffixes.

    Parameters:

      self: (rule object)

      suffixes: (list of allowed suffixes)

      words: (list of words to check)

      iLineNumber: (integer)
    '''
    for word in words:
        for suffix in suffixes:
            if word.endswith(suffix):
                break
        else:
            self.add_violation(iLineNumber)




from vsg.rules import indent_rule


class rule_008(indent_rule):
    '''
    Architecture rule 008 checks for spaces at the beginning of the line for the "end architecture" keywords.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'architecture', '008', 'isEndArchitecture')




from vsg.rules import line_above_rule


class rule_023(line_above_rule):
    '''
    Process rule 023 checks for a blank line above the "end process" keywords.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'process', '023', 'isEndProcess')




from vsg import rule


class rule_016(rule.rule):
    '''
    Process rule 016 checks a process has a label.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '016')
        self.solution = 'Add a label for the process.'
        self.phase = 1
        self.fixable = False   # The user must add the label

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isProcessKeyword and not oLine.isProcessLabel:
            self.add_violation(iLineNumber)




from vsg import rule
from vsg import check
from vsg import fix


class rule_001(rule.rule):
    '''
    Function rule 001 checks for the proper indentation at the beginning of the line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'function'
        self.identifier = '001'
        self.solution = 'Ensure proper indentation.'
        self.phase = 4

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isFunctionKeyword or oLine.isFunctionBegin or oLine.isFunctionEnd or oLine.isFunctionReturn:
            check.indent(self, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.indent(self, oFile.lines[iLineNumber])




from vsg.rules import single_space_after_rule


class rule_003(single_space_after_rule):
    '''
    If rule 003 checks there is a single space between the if keyword and the (.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'if', '003', 'isIfKeyword', 'if')
        self.solution = 'Ensure only a single space exists between the "if" keyword and the (.'




from vsg import rule
from vsg import fix
from vsg import check


class rule_010(rule.rule):
    '''If rule 010 checks for an empty line before the "else" keyword.'''

    def __init__(self):
        rule.rule.__init__(self, 'if', '010')
        self.solution = 'Remove blank line(s) before the "else" keyword.'
        self.phase = 3

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isElseKeyword and not oLine.isIfKeyword:
            check.is_no_blank_line_before(self, oFile, iLineNumber, 'isEndCaseKeyword')

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            fix.remove_blank_lines_above(self, oFile, iLineNumber, 'isEndCaseKeyword')




from vsg import rule

import re


class rule_007(rule.rule):
    '''Whitespace rule 007 checks for spaces after a comma.'''

    def __init__(self):
        rule.rule.__init__(self, 'whitespace', '007')
        self.phase = 2
        self.solution = 'Add a space after the comma.'

    def _analyze(self, oFile, oLine, iLineNumber):
        if re.match('^.*,\S', oLine.line) and not re.match('^.*--.*,\S', oLine.line):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r',(\S)', r', \1', oLine.line))




import re

from vsg import rule
from vsg import utils


class rule_032(rule.rule):
    '''
    Process rule 032 checks the lable for a process is on the same line as the process keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '032')
        self.solution = 'Move process label to the same line as the process.'
        self.phase = 1
        self.fixable = True    # The user must add the label

    def _pre_analyze(self):
        self.label_found = False
        self.label_name = ''
        self.label_line_number = 0

    def _analyze(self, oFile, oLine, iLineNumber):
        if re.match('^\s*\S+\s*:', oLine.lineNoComment) and not oLine.isSignal and not oLine.insidePortMap and not oLine.insideGenericMap and not oLine.insideInstantiation and not oLine.isConstant and not oLine.isVariable:
            self.label_found = True
            self.label_name = utils.extract_label(oLine)[0]
            self.label_line_number = iLineNumber
        if oLine.isProcessKeyword and not oLine.isProcessLabel and self.label_found:
            self.add_violation(iLineNumber)
            self.dFix[iLineNumber] = {}
            self.dFix[iLineNumber]['label_name'] = self.label_name
            self.dFix[iLineNumber]['label_line_number'] = self.label_line_number
        if oLine.isProcessKeyword:
            self.label_found = False
        if oLine.isConcurrentBegin:
            self.label_found = False

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            _remove_label(self, iLineNumber, oFile)
            _add_label(self, iLineNumber, oFile)


def _remove_label(self, iLineNumber, oFile):
    oLine = oFile.lines[self.dFix[iLineNumber]['label_line_number']]
    sLine = oLine.line
    iIndex = sLine.find(':')
    sBlank = ' '*(iIndex + 1)
    oLine.update_line(sBlank + sLine[iIndex+1:])
    if re.match('^\s*$', oLine.line):
        oLine.update_line('')
        oLine.isBlank = True


def _add_label(self, iLineNumber, oFile):
    oLine = oFile.lines[iLineNumber]
    sLine = oLine.line
    oLine.update_line(self.dFix[iLineNumber]['label_name'] + ' : ' + sLine)
    oLine.isProcessLabel = True




from vsg import rule

import re


class rule_030(rule.rule):
    '''
    Process rule 030 checks for single signal declarations on sensitivity list lines.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'process'
        self.identifier = '030'
        self.solution = 'Compact sensitivity list to reduce the number of lines is uses.'
        self.phase = 1
        self.fixable = False

    def _pre_analyze(self):
        self.iCount = 0
        self.sSensitivityList = ''

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.insideSensitivityList:
            self.sSensitivityList += oLine.lineNoComment
            if not re.match('^\s*\)', oLine.lineNoComment):
                self.iCount += 1
        if oLine.isSensitivityListEnd:
            if self.iCount == self.sSensitivityList.count(',') + 1:
                if self.iCount > 1:
                    self.add_violation(iLineNumber)
            self.iCount = 0
            self.sSensitivityList = ''




from vsg.rules import line_above_rule


class rule_018(line_above_rule):
    '''
    Architecture rule 018 checks for a blank line above the "end architecture"
    keywords.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'architecture', '018', 'isEndArchitecture')




def identifier_alignment(self, iLineNumber, lGroup):
    '''
    Checks identifiers in a group of line objects are aligned in the same column.

    Parameters:

      self: (rule object)

      iLineNumber: (integer)

      lGroup: (list of line objects)
    '''
    iMaximumIdentifierColumn = 0
    iMaximumIdentifierLength = 0
    iMaximumKeywordLength = 0
    sViolationRange = str(iLineNumber) + '-' + str(iLineNumber + len(lGroup) - 1)
    self.dFix['violations'][sViolationRange] = {}
    self.dFix['violations'][sViolationRange]['line'] = {}

    for iIndex, oGroupLine in enumerate(lGroup):
        if oGroupLine.isConstant or oGroupLine.isVariable or oGroupLine.isFileKeyword:
            self.dFix['violations'][sViolationRange]['line'][iLineNumber + iIndex] = {}
            iIdentifierAlignment = find_identifier(oGroupLine.line)
            self.dFix['violations'][sViolationRange]['line'][iLineNumber + iIndex]['identifierColumn'] = iIdentifierAlignment

            sKeyword, sIdentifier = parse_keyword_identifier(oGroupLine.line)

            iMaximumIdentifierLength = max(iMaximumIdentifierLength, len(sIdentifier))

            iMaximumKeywordLength = max(iMaximumKeywordLength, len(sKeyword))

            iMaximumIdentifierColumn = max(iMaximumIdentifierColumn, iIdentifierAlignment)

            if not iIdentifierAlignment == iMaximumIdentifierColumn:
                add_range_violation(self, sViolationRange)

    self.dFix['violations'][sViolationRange]['maximumIdentifierColumn'] = iMaximumIdentifierColumn
    self.dFix['violations'][sViolationRange]['maximumIdentifierLength'] = iMaximumIdentifierLength
    self.dFix['violations'][sViolationRange]['maximumKeywordLength'] = iMaximumKeywordLength


def find_identifier(sString):

    fKeywordFound = False
    fSpaceAfterKeywordFound = False
    for iIndex, sChar in enumerate(sString):
        if not sChar.isspace() and not fKeywordFound:
            fKeywordFound = True
        if sChar.isspace() and fKeywordFound:
            fSpaceAfterKeywordFound = True
        if not sChar.isspace() and fSpaceAfterKeywordFound:
            return iIndex


def add_range_violation(self, sViolationRange):
    '''
    Appends a range violation to the violation list if it does not already exist.

    Parameters:

      self: (rule object)

      sViolationRange: (string)
    '''
    if sViolationRange not in self.violations:
        self.add_violation(sViolationRange)


def parse_keyword_identifier(sString):
    lLine = sString.split()
    return lLine[0], lLine[1]



import re


def type_definition(dVars, oLine):
    '''
    Classifies types and subtypes.

    Sets the following line attributes:

      * isTypeKeyword
      * isTypeArrayKeyword
      * insideTypeArray
      * indentLevel
      * isTypeEnd
      * isTypeEnumeratedKeyword
      * insideTypeEnumerated
      * isTypeRecordKeyword
      * insideTypeRecord
      * isTypeArrayEnd
      * isTypeRecordEnd
      * isTypeEnumeratedEnd
    '''
    if re.match('^\s*type\s+\w+\s+is\s+array', oLine.line, re.IGNORECASE):
        oLine.isTypeKeyword = True
        oLine.isTypeArrayKeyword = True
        oLine.insideTypeArray = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
    elif re.match('^\s*type\s.*range', oLine.line, re.IGNORECASE):
        oLine.isTypeKeyword = True
        oLine.isTypeEnd = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
    elif re.match('^\s*type\s+\w+\s+is\s+\(', oLine.lineNoComment, re.IGNORECASE):
        oLine.isTypeKeyword = True
        oLine.isTypeEnumeratedKeyword = True
        oLine.insideTypeEnumerated = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 1
    elif re.match('^\s*type', oLine.line, re.IGNORECASE):
        if not oLine.insideEntity and not oLine.insideComponent and \
           not oLine.insideInstantiation and not oLine.insideConcurrent and \
           not (oLine.insideProcess and not oLine.isProcessDeclarative):
            oLine.isTypeKeyword = True
            oLine.indentLevel = dVars['iCurrentIndentLevel']

    if re.match('^.*\srecord', oLine.lineNoComment, re.IGNORECASE) and not \
       re.match('^.*end\s+record', oLine.lineNoComment, re.IGNORECASE) and not \
       re.match('^.*record\w', oLine.lineNoComment, re.IGNORECASE) and not \
       oLine.isLibraryUse:
        oLine.isTypeRecordKeyword = True
        oLine.insideTypeRecord = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 1

    assign_array_attributes(oLine)
    assign_record_attributes(dVars, oLine)
    assign_enumerated_attributes(dVars, oLine)


def assign_array_attributes(oLine):
    if oLine.insideTypeArray:
        if ';' in oLine.line:
            oLine.isTypeArrayEnd = True
            oLine.isTypeEnd = True


def assign_record_attributes(dVars, oLine):
    if oLine.insideTypeRecord:
        if not oLine.isTypeRecordKeyword and not oLine.isBlank:
            oLine.indentLevel = dVars['iCurrentIndentLevel']
        if re.match('^\s*end\s+record', oLine.line, re.IGNORECASE):
            oLine.isTypeRecordEnd = True
            oLine.isTypeEnd = True
            dVars['iCurrentIndentLevel'] -= 1
            oLine.indentLevel = dVars['iCurrentIndentLevel']


def assign_enumerated_attributes(dVars, oLine):
    if oLine.insideTypeEnumerated:
        if not oLine.isTypeEnumeratedKeyword and not oLine.isBlank:
            oLine.indentLevel = dVars['iCurrentIndentLevel']
        if ';' in oLine.line:
            oLine.isTypeEnumeratedEnd = True
            oLine.isTypeEnd = True

            dVars['iCurrentIndentLevel'] -= 1
            if re.match('^\s*\)\s*;', oLine.line):
                oLine.indentLevel = dVars['iCurrentIndentLevel']




from vsg import rule
from vsg import line

import re


class rule_010(rule.rule):
    '''
    Process rule 010 checks the "begin" keyword is on it's own line.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '010')
        self.solution = 'Place "begin" keyword on seperate line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isProcessBegin and not re.match('^\s*begin', oLine.line, re.IGNORECASE):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub('begin', ' '*len('begin'), oLine.line, 1, flags=re.IGNORECASE))
            oLine.isProcessBegin = False
            oFile.lines.insert(iLineNumber + 1, line.line('  begin'))
            oFile.lines[iLineNumber + 1].isProcessBegin = True
            oFile.lines[iLineNumber + 1].indentLevel = oLine.indentLevel




from vsg import rule
from vsg import utils

import re


class rule_006(rule.rule):
    '''
    Architecture rule 006 checks if the "is" keyword is on the same line as the "architecture" keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'architecture', '006')
        self.solution = 'Ensure "is" keyword is on the same line as the "architecture" keyword.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isArchitectureKeyword and \
           re.match('^\s*architecture\s+\w+\s+of\s+\w+', oLine.line, re.IGNORECASE) and \
           not re.match('^\s*architecture\s+\w+\s+of\s+\w+\s+is', oLine.line, re.IGNORECASE):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'^(\s*architecture\s+\w+\s+of\s+\w+)', r'\1 is', oLine.line, re.IGNORECASE))
            utils.search_for_and_remove_keyword(oFile, iLineNumber, 'is')



import re


def constant(dVars, oLine, oLinePrevious):

    if re.match('^\s*constant', oLine.lineLower) and \
       not oLine.insideFunction and not oLine.insideProcedure and \
       not oLine.insideConcurrent:
        oLine.isConstant = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        oLine.insideConstant = True
        dVars['iCurrentIndentLevel'] += 1
        if re.match('^.*:=\s*\(', oLine.lineNoComment):
            dVars['fConstantArray'] = True
    if oLine.insideConstant:
        if ';' in oLine.line:
            dVars['iCurrentIndentLevel'] -= 1
            oLine.indentLevel = dVars['iCurrentIndentLevel']
            oLine.isConstantEnd = True
            if dVars['fConstantArray']:
                oLine.isConstantArray = True
            dVars['fConstantArray'] = False
        elif not oLine.isConstant:
            if re.match('^\s*\(', oLine.line):
                oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
                dVars['fConstantArray'] = True
                oLinePrevious.isConstantArray = True
            else:
                oLine.indentLevel = dVars['iCurrentIndentLevel']

    if dVars['fConstantArray']:
        oLine.isConstantArray = True





from vsg import rule
from vsg import check
from vsg import fix


class rule_004(rule.rule):
    '''
    Variable assignment rule 004 ensures the alignment of multiline variable_assignment statements.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'variable_assignment', '004')
        self.solution = 'Align with space after the ":=" keyword.'
        self.phase = 5

    def analyze(self, oFile):
        for iLineNumber, oLine in enumerate(oFile.lines):
            if self._is_vsg_off(oLine):
                continue
            if oLine.isVariableAssignment and oLine.isVariableAssignmentEnd:
                continue
            if oLine.isVariableAssignment:
                iAlignmentColumn = oLine.line.find(':=') + len(':= ')
                continue
            if oLine.insideVariableAssignment and not oLine.isComment:
                check.multiline_alignment(self, iAlignmentColumn, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.dFix['violations']:
            fix.multiline_alignment(self, oFile, iLineNumber)




from vsg import rule
from vsg import fix
from vsg import check
from vsg import utils


class rule_004(rule.rule):
    '''
    Signal rule 004 checks the signal name is lowercase.
    '''
    def __init__(self, name='signal', identifier='004'):
        rule.rule.__init__(self, name, identifier)
        self.phase = 6
        self.solution = 'Change signal name to lowercase.'

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isSignal:
            for sWord in utils.extract_class_identifier_list(oLine):
                check.is_lowercase(self, sWord, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            for sWord in utils.extract_class_identifier_list(oLine):
                fix.lower_case(oLine, sWord)




from vsg.rules import multiple_spaces_after_rule


class rule_003(multiple_spaces_after_rule):
    '''
    Signal rule 003 checks there is a single space after the "signal" keyword.
    '''

    def __init__(self):
        multiple_spaces_after_rule.__init__(self, 'signal', '003', 'isSignal', 'signal')



import re


def process(dVars, oLine, lLines):
    classify_process_keyword(dVars, oLine)
    if oLine.insideProcess:
        classify_process_sensitivity_list(dVars, oLine)
        classify_process_begin_keyword(dVars, oLine)
        classify_process_end_keyword(dVars, oLine)
        classify_clock_process(dVars, oLine, lLines)


def classify_process_keyword(dVars, oLine):
    if re.match('^\s*process[:|\s|(]', oLine.lineLower):
        oLine.isProcessKeyword = True
        oLine.insideProcess = True
        dVars['iProcessIndentLevel'] = dVars['iCurrentIndentLevel']
        oLine.indentLevel = dVars['iCurrentIndentLevel']
    if re.match('^\s*process\s*$', oLine.lineNoComment):
        oLine.isProcessKeyword = True
        oLine.insideProcess = True
        dVars['iProcessIndentLevel'] = dVars['iCurrentIndentLevel']
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 1
    if re.match('^\s*\S+\s*:\s*process', oLine.lineLower) and not oLine.isComment:
        oLine.isProcessKeyword = True
        oLine.insideProcess = True
        dVars['iProcessIndentLevel'] = dVars['iCurrentIndentLevel']
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        oLine.isProcessLabel = True
    if re.match('^\s*\S+\s*:\s*process\s*$', oLine.lineNoComment) and not oLine.isComment:
        dVars['iProcessIndentLevel'] = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 1


def classify_process_sensitivity_list(dVars, oLine):
    # Check sensitivity list
    if '(' in oLine.line and \
       not oLine.insideProcedure and \
       not oLine.insideSensitivityList and \
       not dVars['fFoundProcessBegin'] and \
       not dVars['SensitivityListFound'] and \
       not oLine.isVariable:
        oLine.isSensitivityListBegin = True
        oLine.insideSensitivityList = True
        dVars['SensitivityListFound'] = True
    if oLine.insideSensitivityList:
        dVars['iOpenParenthesis'] += oLine.line.count('(')
        dVars['iCloseParenthesis'] += oLine.line.count(')')
        if dVars['iOpenParenthesis'] == dVars['iCloseParenthesis']:
            oLine.isSensitivityListEnd = True
            dVars['iOpenParenthesis'] = 0
            dVars['iCloseParenthesis'] = 0
            dVars['iCurrentIndentLevel'] += 1


def classify_process_begin_keyword(dVars, oLine):
    if not oLine.insideProcedure:
        if re.match('^.*\s+begin', oLine.lineNoComment, flags=re.IGNORECASE) or re.match('^\s*begin', oLine.lineLower):
            oLine.indentLevel = dVars['iProcessIndentLevel']
            oLine.isProcessBegin = True
            dVars['fFoundProcessBegin'] = True


def classify_process_end_keyword(dVars, oLine):
    if re.match('^\s*end\s+process', oLine.lineLower):
        oLine.indentLevel = dVars['iProcessIndentLevel']
        oLine.isEndProcess = True
        dVars['fFoundProcessBegin'] = False
        dVars['iCurrentIndentLevel'] = dVars['iCurrentIndentLevel'] - 1
        dVars['SensitivityListFound'] = False

def classify_clock_process(dVars, oLine, lLines):
    if re.match('^\s*[elsif|if]\s*.*\'event\s+and\s+\w+\s*=\s*\'[0-1]\'', oLine.lineNoComment, flags=re.IGNORECASE):
        oLine.insideClockProcess = True
        classify_reset_process(oLine, lLines)
    if re.match('^\s*[elsif|if]\s*.*rising_edge', oLine.lineNoComment, flags=re.IGNORECASE):
        oLine.insideClockProcess = True
        classify_reset_process(oLine, lLines)
    if re.match('^\s*[elsif|if]\s*.*falling_edge', oLine.lineNoComment, flags=re.IGNORECASE):
        oLine.insideClockProcess = True
        classify_reset_process(oLine, lLines)

def classify_reset_process(oLine, lLines):
    if re.match('^\s*if', oLine.lineNoComment, flags=re.IGNORECASE):
        return

    iIndex = len(lLines) - 1

    while lLines[iIndex].isIfKeyword is False:
        lLines[iIndex].insideResetProcess = True
        iIndex = iIndex - 1
    lLines[iIndex].insideResetProcess = True




from vsg import rule
from vsg import fix

import re


class rule_019(rule.rule):
    '''
    Architecture rule 019 checks the "of" keyword is lower case.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'architecture', '019')
        self.solution = 'Change "of" keyword to lowercase.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isArchitectureKeyword and \
           re.match('^\s*\w+\s+\w+\s+of', oLine.lineLower) and \
           not re.match('^\s*\w+\s+\w+\s+of', oLine.line):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(oFile.lines[iLineNumber], 'of')




from vsg import rule
from vsg import fix

import re


class rule_005(rule.rule):
    '''
    Constant rule 005 checks there is a single space after the colon.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'constant'
        self.identifier = '005'
        self.solution = 'Ensure only a single space after the colon.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isConstant:
            if not re.match('^\s*constant\s+\w+\s*:\s\w', oLine.lineLower):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], ':')




from vsg.rules import case_rule
from vsg import utils


class rule_010(case_rule):
    '''
    Port rule 010 checks port names have proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'port', '010', 'isPortDeclaration')
        self.case = 'upper'
        self.solution = 'Change port name to '

    def _extract(self, oLine):
        return utils.extract_port_name(oLine)




from vsg import rule

import re
import copy


class rule_021(rule.rule):
    '''
    Instantiation rule 021 checks multiple port assignments on the same line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '021'
        self.solution = 'Move multiple port assignments to their own lines.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationPortAssignment:
            if re.match('^\s*\S+\s*=>\s*.*\s*,\s*\S+\s*=>', oLine.line):
                sTemp = re.match('(^\s*\S+\s*=>\s*.*\s*),\s*\S+\s*=>', oLine.line).group(0)
                if sTemp.count('(') == sTemp.count(')'):
                    self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            oLine = oFile.lines[iLineNumber]
            iNumberOfPorts = oLine.line.count(',')
            # Replicate ports ###
            for iIndex in range(1, iNumberOfPorts):
                oFile.lines.insert(iLineNumber, copy.deepcopy(oLine))
            # Split ports
            for iIndex in range(0, iNumberOfPorts):
                oLine = oFile.lines[iLineNumber + iIndex]
                lPorts = oLine.line.split(',')
                if iIndex == iNumberOfPorts - 1 and oFile.lines[iLineNumber + iIndex + 1].isInstantiationPortEnd:
                    oLine.update_line(lPorts[iIndex])
                else:
                    oLine.update_line(lPorts[iIndex] + ',')




from vsg import rule
from vsg import utils

import re


class rule_016(rule.rule):
    '''
    Port rule 016 checks for a port definition on the same line as the port keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'port', '016')
        self.solution = 'Move port definition to it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPortKeyword and re.match('^\s*port\s*\(\s*\S+\s*:', oLine.lineLower):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            utils.copy_line(oFile, iLineNumber)
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(extract_port_keyword(oLine.line))
            oLine = oFile.lines[iLineNumber + 1]
            oLine.update_line('    ' + extract_signal_definition(oLine.line))
            oLine.isPortKeyword = False
            oLine.isPortDeclaration = True
            oLine.insidePortMap = True
            oLine.indentLevel = oFile.lines[iLineNumber].indentLevel + 1

def extract_port_keyword(sString):
    return sString.split('(')[0] + ' ('

def extract_signal_definition(sString):
    lString = sString.split('(')
    return '('.join(lString[1:])




from vsg import rule
from vsg import fix

import re


class rule_006(rule.rule):
    '''Component rule 006 checks the "is" keyword is lower case.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'component'
        self.identifier = '006'
        self.solution = 'Change "is" keyword to lowercase.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isComponentDeclaration and re.match('^\s*\S+\s+\S+\s\s*is', oLine.line, re.IGNORECASE):
            if not re.match('^\s*\S+\s+\S+\s\s*is', oLine.line):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(oFile.lines[iLineNumber], 'is')




from vsg import rule
from vsg import check
from vsg import fix


class rule_003(rule.rule):
    '''
    Concurrent rule 003 checks the alignment of multiline concurrent assignments.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'concurrent'
        self.identifier = '003'
        self.solution = 'Align first character in row to the column of text one space after the <=.'
        self.phase = 6
        self.iAlignmentColumn = 0

    def _pre_analyze(self):
        self.iAlignmentColumn = 0

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.insideConcurrent:
            if oLine.isConcurrentBegin:
                if not oLine.isEndConcurrent:
                    self.iAlignmentColumn = oLine.line.find('<') + 3
            else:
                check.multiline_alignment(self, self.iAlignmentColumn, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.dFix['violations']:
            fix.multiline_alignment(self, oFile, iLineNumber)




from vsg import rule
from vsg import check
from abc import abstractmethod


class prefix_rule(rule.rule):
    '''
    Checks for identifiers prefixes. This rule can not fix!

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    sTrigger : string
       The line attribute the rule applies to.

    Attribute
    ----------

    self.phase : integer = 7
       Sets the phase the rule will run in.

    self.solution : string = None
       Instructions on how to fix the violation.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None):
        rule.rule.__init__(self, name, identifier)
        self.phase = 7
        self.solution = None
        self.sTrigger = sTrigger
        self.prefixes = None
        self.fixable = False  # The user will have to fix any desired prefixes.
        self.disable = True
        self.configuration.append('prefixes')

    @abstractmethod
    def _extract(self, oLine):
        pass

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.sTrigger]:
            words = self._extract(oLine)

            check.has_prefix(self, self.prefixes, words, iLineNumber)

    def _get_solution(self, iLineNumber):
        return self.solution + ' identifiers should start with one of ' + str(self.prefixes) + ' prefixes.'



from vsg.rules import indent_rule


class rule_001(indent_rule):
    '''
    Checks the indent level of the "subtype" keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self)
        # These are filled out when creating a new rule
        self.name = 'subtype'
        self.identifier = '001'
        self.sTrigger = 'isSubtypeKeyword'




import re

from vsg import rule


class rule_014(rule.rule):
    '''
    Constant rule 014 checks the indent of multiline constants that are not arrays.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'constant'
        self.identifier = '014'
        self.solution = 'Align with := keyword on constant declaration line.'
        self.phase = 5

    def _pre_analyze(self):
        self.alignmentColumn = 0
        self.fKeywordFound = False

    def _analyze(self, oFile, oLine, iLineNumber):
        if not oLine.isConstantArray and oLine.insideConstant:
            if oLine.isConstant and ':=' in oLine.line:
                self.alignmentColumn = oLine.line.index(':=') + len(':= ')
                self.fKeywordFound = True
            elif not oLine.isConstant and self.fKeywordFound:
                sMatch = ' ' * self.alignmentColumn
                if not re.match('^' + sMatch + '\w', oLine.line):
                    self.add_violation(iLineNumber)
                    self.dFix['violations'][iLineNumber] = self.alignmentColumn
            if oLine.isConstantEnd:
                self.fKeywordFound = False

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            sLine = oFile.lines[iLineNumber].line
            sNewLine = ' ' * self.dFix['violations'][iLineNumber] + sLine.strip()
            oFile.lines[iLineNumber].update_line(sNewLine)




from vsg.rules import case_rule
from vsg import utils


class rule_017(case_rule):
    '''
    Port rule 017 checks the "port" keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'port', '017', 'isPortKeyword')
        self.solution = 'Change "port" keyword to '

    def _extract(self, oLine):
        return utils.extract_first_keyword(oLine)


import re


def architecture(self, dVars, oLine):

    classify_architecture_keyword(self, dVars, oLine)
    if oLine.insideArchitecture:
        classify_begin_keyword(dVars, oLine)
        classify_end_keyword(dVars, oLine)


def classify_architecture_keyword(self, dVars, oLine):
    if re.match('^\s*architecture', oLine.lineLower) and not oLine.insideArchitecture:
        self.hasArchitecture = True
        oLine.isArchitectureKeyword = True
        oLine.insideArchitecture = True
        oLine.indentLevel = 0
        dVars['iCurrentIndentLevel'] = 1


def classify_begin_keyword(dVars, oLine):
        if re.match('^\s*begin', oLine.lineLower) and not oLine.insideFunction and \
           not oLine.insideProcess and not oLine.insideGenerate and \
           not oLine.insideProcedure and not oLine.insideBlock:
            oLine.isArchitectureBegin = True
            oLine.indentLevel = 0
            dVars['iCurrentIndentLevel'] = 1


def classify_end_keyword(dVars, oLine):
    if not oLine.insideProcess and not oLine.insideCase and \
       not oLine.insideComponent and not oLine.insideGenerate and \
       not oLine.insideFunction and not oLine.insideForLoop and \
       not oLine.insideWhileLoop and not oLine.insideTypeRecord and \
       not oLine.insideProcedure and not oLine.insideBlock and \
       not oLine.insideConcurrent and not oLine.insideInstantiation:
        if re.match('^\s*end', oLine.lineLower) and not \
           re.match('^\s*end\S+\s*:', oLine.lineLower) and not \
           re.match('^\s*end_', oLine.lineLower):
            oLine.isEndArchitecture = True
            oLine.indentLevel = 0
            dVars['iCurrentIndentLevel'] = 0




from vsg import rule
from vsg import check
from vsg import fix


class single_space_after_character_rule(rule.rule):
    '''
    Single space after rule checks for a single space after a user specified keyword.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None, sWord=None):
        rule.rule.__init__(self, name=name, identifier=identifier)
        self.phase = 2
        self.sTrigger = sTrigger
        self.sWord = sWord
        self.solution = None

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.sTrigger]:
            check.is_single_space_after_character(self, self.sWord, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            fix.enforce_one_space_after_word(self, oLine, self.sWord)




from vsg import rule
from vsg import utils
from vsg import fix
from vsg import check


class rule_004(rule.rule):
    '''
    Process rule 004 checks the "begin" keyword is lower case.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'process'
        self.identifier = '004'
        self.solution = 'Lowercase the "begin" keyword.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isProcessBegin:
            check.is_lowercase(self, utils.get_first_word(oLine), iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(oFile.lines[iLineNumber], 'begin')



import re


def generate(dVars, oLine, oLinePrevious):

    if re.match('^\s*\w+\s*:\s*if', oLine.lineLower) or re.match('^\s*\w+\s*:\s*for\s.*\sgenerate', oLine.lineLower):
        oLine.insideGenerate = True
        dVars['iGenerateLevel'] += 1
    if re.match('^\s*\w+\s*:\s*--', oLinePrevious.line) or re.match('^\s*\w+\s*:\s*$', oLinePrevious.line):
        if re.match('^\s*if\s+.*\sgenerate', oLine.lineLower) or re.match('^\s*for\s.*\sgenerate', oLine.lineLower):
            oLinePrevious.insideGenerate = True
            oLinePrevious.isGenerateLabel = True
            oLine.insideGenerate = True
            dVars['iGenerateLevel'] += 1
    if oLine.insideGenerate:
        if re.match('^\s*\w+\s*:\s*if[\s|(].*[\s|)]generate', oLine.lineLower) or re.match('^\s*\w+\s*:\s*for\s.*\sgenerate', oLine.lineLower):
            oLine.isGenerateKeyword = True
            oLine.isGenerateLabel = True
            oLine.indentLevel = dVars['iCurrentIndentLevel']
            dVars['iCurrentIndentLevel'] += 1
        if re.match('^\s*if[\s|(].*[\s|)]generate', oLine.lineLower) or re.match('^\s*for\s.*\sgenerate', oLine.lineLower):
            oLine.isGenerateKeyword = True
            oLine.indentLevel = dVars['iCurrentIndentLevel']
            dVars['iCurrentIndentLevel'] += 1
        if re.match('^\s*begin', oLine.lineLower):
            oLine.isGenerateBegin = True
            oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
        if re.match('^\s*end\s+generate', oLine.lineLower):
            oLine.isGenerateEnd = True
            dVars['iCurrentIndentLevel'] -= 1
            oLine.indentLevel = dVars['iCurrentIndentLevel']
            dVars['iGenerateLevel'] -= 1




from vsg import rule
from vsg import line
from vsg import utils


class rule_007(rule.rule):
    '''
    Instantiation rule 007 checks the closing ) for the port map is on it's own line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '007'
        self.solution = 'Place closing ); on it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationPortEnd and oLine.isInstantiationPortAssignment:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            oFile.lines[iLineNumber].line = utils.remove_closing_parenthesis_and_semicolon(oFile.lines[iLineNumber].line)
            oFile.lines[iLineNumber].isInstantiationPortEnd = False
            oFile.lines.insert(iLineNumber + 1, line.line('  );'))
            oFile.lines[iLineNumber + 1].isInstantiationPortAssignement = False
            oFile.lines[iLineNumber + 1].isInstantiationPortEnd = True
            oFile.lines[iLineNumber + 1].insideInstantiation = True
            if oFile.lines[iLineNumber].isInstantiationPortKeyword:
                oFile.lines[iLineNumber + 1].indentLevel = oFile.lines[iLineNumber].indentLevel
            else:
                oFile.lines[iLineNumber + 1].indentLevel = oFile.lines[iLineNumber].indentLevel - 1




from vsg import rule
from vsg import fix

import re


class rule_005(rule.rule):
    '''
    Function rule 005 checks the "function" keyword is lower case.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'function'
        self.identifier = '005'
        self.solution = 'Lowercase the "function" keyword.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isFunctionKeyword and \
           not re.match('^\s*function', oLine.line) and \
           not re.match('^\s*impure\s+function', oLine.line) and \
           not re.match('^\s*pure\s+function', oLine.line):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(oFile.lines[iLineNumber], 'function')




from vsg.rules import case_rule
from vsg import utils


class rule_013(case_rule):
    '''
    Entity rule 013 checks the "is" keyword has proper case in type definitions.
    '''

    def __init__(self):
        case_rule.__init__(self, 'type', '013', 'isTypeKeyword')
        self.solution = 'Change "is" keyword to '

    def _extract(self, oLine):
        return utils.extract_word(oLine, 'is')




from vsg.rules import case_rule
from vsg import utils


class rule_002(case_rule):
    '''
    Signal rule 002 checks the "variable" keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'variable', '002', 'isVariable')
        self.solution = 'Change "variable" keyword to '

    def _extract(self, oLine):
        return utils.extract_class_name(oLine)




from vsg.rules import case_rule
from vsg import utils


class rule_016(case_rule):
    '''
    Entity rule 016 checks the when keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'case', '016', 'isCaseWhenKeyword')
        self.solution = 'Change "when" keyword to '

    def _extract(self, oLine):
        return utils.extract_word(oLine, 'when')




from vsg.rules import single_space_after_rule


class rule_003(single_space_after_rule):
    '''
    Attribute rule 003 checks for a single space after the **attribute** keyword.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'attribute', '003', 'isAttributeKeyword', 'attribute')
        self.solution = 'Ensure a single space after the "attribute" keyword.'




from vsg.rules import single_space_before_character_rule


class rule_006(single_space_before_character_rule):
    '''
    Constant rule 006 checks there is at least a single space before the colon.
    '''

    def __init__(self):
        single_space_before_character_rule.__init__(self, 'constant', '006', 'isConstant', ':')
        self.solution = 'Add a single space before the colon.'



import re


def when(dVars, oLine, oLinePrevious):

    if re.match('^\s*\S+\s*<=\s*\S+\swhen', oLine.lineNoComment, re.IGNORECASE):
        oLine.insideWhen = True

    if oLine.insideWhen or (oLinePrevious.insideWhen and not oLinePrevious.isWhenEnd):
        if re.match('^.*\swhen', oLine.lineNoComment, re.IGNORECASE):
            oLine.isWhenKeyword = True
        if re.match('^.*\selse', oLine.lineNoComment, re.IGNORECASE):
            oLine.isWhenElseKeyword = True
        if re.match('^.*;', oLine.lineNoComment, re.IGNORECASE):
            oLine.isWhenEnd = True




from vsg.rules import single_space_after_rule


class rule_002(single_space_after_rule):
    '''
    Component rule 002 checks for a single space after the "component" keyword.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'component', '002', 'isComponentDeclaration', 'component')
        self.solution = 'Remove extra spaces after "component" keyword.'




from vsg import rule

import re


class rule_011(rule.rule):
    '''
    Generate rule 011 checks the "end generate" has a label.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'generate', '011')
        self.solution = 'Add a label for the "end generate".'
        self.phase = 1

    def _pre_analyze(self):
        self.sGenerateName = ''

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isGenerateLabel:
           self.sGenerateName = oLine.line.lstrip().split(':')[0]
        if oLine.isGenerateEnd and not re.match('^\s*\S+\s+\S+\s+\S+', oLine.line):
            self.add_violation(iLineNumber)
            self.dFix['violations'][iLineNumber] = self.sGenerateName

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            sLine = oLine.line
            iIndex = oLine.lineLower.find('generate') + len('generate')
            oLine.update_line(sLine[:iIndex] + ' ' + self.dFix['violations'][iLineNumber] + sLine[iIndex:])




from vsg import rule
from vsg import check
from vsg import fix


class rule_001(rule.rule):
    '''
    Comment rule 001 checks consecutive comment lines above a "use" keyword
    in a library declaration "use" keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'comment'
        self.identifier = '001'
        self.solution = 'Align comment with "use" keyword.'
        self.phase = 4

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isLibraryUse:
            check.indent_of_comments_above(self, oFile, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oFile.lines[iLineNumber].indentLevel = self.dFix['violations'][iLineNumber]
            fix.indent(self, oFile.lines[iLineNumber])




from vsg import rule

import re


class rule_027(rule.rule):
    '''
    Instantiation rule 027 checks the **entity** keyword is lowercase in direct instantiations.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '027'
        self.solution = 'Uppercase "entity" keyword.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isDirectInstantiationDeclaration and not re.match('^\s*\w+\s*:\s*entity', oLine.line):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub('entity', 'entity', oLine.line, 1, re.IGNORECASE))




from vsg import rule
from vsg import check
from vsg import fix
from vsg import line


class rule_031(rule.rule):
    '''Process rule 031 checks for alignment of identifiers and colons in the process declarative region.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'process'
        self.identifier = '031'
        self.phase = 5
        self.solution = 'Align the first character of each identifier and align colons.'

    def _pre_analyze(self):
        self.lGroup = []
        self.fGroupFound = False
        self.iStartGroupIndex = None

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isProcessKeyword and not self.fGroupFound:
            self.fGroupFound = True
            self.iStartGroupIndex = iLineNumber
        if self.fGroupFound:
            if oLine.isConstant or oLine.isVariable or oLine.isFileKeyword:
                self.lGroup.append(oLine)
            else:
                self.lGroup.append(line.blank_line())
        if oLine.isProcessBegin and self.fGroupFound:
            self.fGroupFound = False
            check.identifier_alignment(self, self.iStartGroupIndex, self.lGroup)
            check.keyword_alignment(self, self.iStartGroupIndex, ':', self.lGroup)
            self.lGroup = []
            self.iStartGroupIndex = None

    def _fix_violations(self, oFile):
        fix.identifier_alignment(self, oFile)




from vsg.rules import keyword_alignment_rule


class rule_015(keyword_alignment_rule):
    '''
    Instantiation rule 015 ensures the alignment of the => operator for every generic in the instantiation.
    '''

    def __init__(self):
        keyword_alignment_rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '015'
        self.solution = 'Inconsistent alignment of "=>" in generic assignments of instantiation.'
        self.sKeyword = '=>'
        self.sStartGroupTrigger = 'isInstantiationGenericKeyword'
        self.sEndGroupTrigger = 'isInstantiationGenericEnd'
        self.sLineTrigger = 'isInstantiationGenericAssignment'



'''
This module provides functions for rules to use.
'''
import copy
import re

try:
    from vsg import line
except ImportError:
    import line


def copy_line(oFile, iLineNumber):
    '''
    Creates a copy of the line at iLineNumber and inserts it below iLineNumber.

    Parameters:

      oFile: (vhdlFile object)

      iLineNumber: (integer)
    '''
    oFile.lines.insert(iLineNumber + 1, copy.deepcopy(oFile.lines[iLineNumber]))


def split_line_after_word(oFile, iLineNumber, sWord):
    '''
    Splits the line after the word given and inserts it after the current line.

    Parameters:

      oFile: (vhdlFile object)

      iLineNumber: (integer)

      sWord: (string)
    '''
    copy_line(oFile, iLineNumber)
    oLine = oFile.lines[iLineNumber]
    iIndex = oLine.line.find(sWord) + len(sWord)
    oLine.update_line(oLine.line[:iIndex])
    oLine = oFile.lines[iLineNumber + 1]
    oLine.update_line(oLine.line[iIndex:])


def split_line_before_word(oFile, iLineNumber, sWord):
    '''
    Splits the line before the word given and inserts it after the current line.

    Parameters:

      oFile: (vhdlFile object)

      iLineNumber: (integer)

      sWord: (string)
    '''
    copy_line(oFile, iLineNumber)
    oLine = oFile.lines[iLineNumber]
    iIndex = oLine.line.find(sWord)
    oLine.update_line(oLine.line[:iIndex])
    oLine = oFile.lines[iLineNumber + 1]
    oLine.update_line(oLine.line[iIndex:])


def get_word(oLine, iIndex):
    '''
    Returns a word from a line at iIndex.

    Parameters:

      oLine: (line object)

      iIndex: (integer)

    Returns: (string)
    '''
    return oLine.line.split()[iIndex]


def get_first_word(oLine):
    '''
    Returns the first word from a line at iIndex.

    Parameters:

      oLine: (line object)

    Returns: (string)
    '''
    return get_word(oLine, 0)


def change_word(oLine, sWord, sNewWord, iMax=1):
    '''
    Changes one word in the line to another.

    Parameters:

      oLine: (line object)

      sWord: (string)

      sNewWord: (string)
    '''
    sLine = oLine.line
    tLine = re.subn(r'\b' + sWord + r'\b', sNewWord, sLine, iMax, flags=re.IGNORECASE)
    sLine = tLine[0]
    if tLine[1] == 0:
        tLine = re.subn(' ' + sWord + ';', sNewWord + ';', sLine, iMax, flags=re.IGNORECASE)
        sLine = tLine[0]
    if tLine[1] == 0:
        tLine = re.subn(sWord + '$', sNewWord, sLine, iMax, flags=re.IGNORECASE)
        sLine = tLine[0]

#    tLine = re.subn(r' ' + sWord + '([\s|;|:|\(])', r' ' + sNewWord + r'\1', sLine, iMax, flags=re.IGNORECASE)
#    if tLine[1] == 0:
#    if tLine[1] == 0:
#        tLine = re.subn('^' + sWord + '$', sNewWord, sLine, iMax, flags=re.IGNORECASE)
#        sLine = tLine[0]
#    if tLine[1] == 0:
#        tLine = re.subn('^' + sWord + ' ', sNewWord + ' ', sLine, iMax, flags=re.IGNORECASE)
#        sLine = tLine[0]
#    if tLine[1] == 0:
#        tLine = re.subn('\(' + sWord + '\)', '(' + sNewWord + ')', sLine, iMax, flags=re.IGNORECASE)
#        sLine = tLine[0]
#    if tLine[1] == 0:
#        tLine = re.subn('\(' + sWord + '\s', '(' + sNewWord + ' ', sLine, iMax, flags=re.IGNORECASE)
#        sLine = tLine[0]
#    if tLine[1] == 0:
#        tLine = re.subn(sWord + ',', sNewWord + ',', sLine, iMax, flags=re.IGNORECASE)
#        sLine = tLine[0]
    sLine = tLine[0]
    oLine.update_line(sLine)


def remove_text_after_word(sKeyword, sWord):
    '''
    Removes all text after a keyword.

    Parameters:

      sKeyword: (string)

      sWord: (string)
    '''
    if sKeyword in sWord:
        sWord = sWord[:sWord.find(sKeyword)]
    return sWord


def is_vhdl_keyword(sWord):
    '''
    Returns True if given word is a VHDL keyword.

    Returns False if given word is not a VHDL keyword.

    Parameters:

      sWord: (string)

    Returns: (boolean)
    '''
    lKeywords = []
    lKeywords.append('std_logic')
    lKeywords.append('std_logic_vector')
    lKeywords.append('integer')
    lKeywords.append('signed')
    lKeywords.append('unsigned')
    lKeywords.append('natural')
    lKeywords.append('std_ulogic')
    lKeywords.append('event')
    lKeywords.append('is')
    lKeywords.append('end')
    lKeywords.append('entity')
    lKeywords.append('architecture')
    lKeywords.append('begin')
    lKeywords.append('process')
    lKeywords.append('downto')
    lKeywords.append('others')
    lKeywords.append('signal')
    lKeywords.append('else')
    lKeywords.append('when')
    lKeywords.append('port')
    lKeywords.append('map')
    lKeywords.append('generic')
    lKeywords.append('abs')
    lKeywords.append('library')
    lKeywords.append('postponed')
    lKeywords.append('srl')
    lKeywords.append('access')
    lKeywords.append('linkage')
    lKeywords.append('procedure')
    lKeywords.append('subtype')
    lKeywords.append('after')
    lKeywords.append('elsif')
    lKeywords.append('literal')
    lKeywords.append('then')
    lKeywords.append('alias')
    lKeywords.append('loop')
    lKeywords.append('pure')
    lKeywords.append('to')
    lKeywords.append('all')
    lKeywords.append('range')
    lKeywords.append('transport')
    lKeywords.append('and')
    lKeywords.append('exit')
    lKeywords.append('mod')
    lKeywords.append('record')
    lKeywords.append('type')
    lKeywords.append('file')
    lKeywords.append('nand')
    lKeywords.append('register')
    lKeywords.append('unaffected')
    lKeywords.append('array')
    lKeywords.append('for')
    lKeywords.append('new')
    lKeywords.append('reject')
    lKeywords.append('units')
    lKeywords.append('assert')
    lKeywords.append('function')
    lKeywords.append('next')
    lKeywords.append('rem')
    lKeywords.append('until')
    lKeywords.append('attribute')
    lKeywords.append('generate')
    lKeywords.append('nor')
    lKeywords.append('report')
    lKeywords.append('use')
    lKeywords.append('not')
    lKeywords.append('return')
    lKeywords.append('variable')
    lKeywords.append('block')
    lKeywords.append('group')
    lKeywords.append('null')
    lKeywords.append('rol')
    lKeywords.append('wait')
    lKeywords.append('body')
    lKeywords.append('guarded')
    lKeywords.append('of')
    lKeywords.append('ror')
    lKeywords.append('buffer')
    lKeywords.append('if')
    lKeywords.append('on')
    lKeywords.append('select')
    lKeywords.append('while')
    lKeywords.append('bus')
    lKeywords.append('impure')
    lKeywords.append('open')
    lKeywords.append('severity')
    lKeywords.append('with')
    lKeywords.append('case')
    lKeywords.append('in')
    lKeywords.append('or')
    lKeywords.append('shared')
    lKeywords.append('xnor')
    lKeywords.append('component')
    lKeywords.append('inertial')
    lKeywords.append('xor')
    lKeywords.append('configuration')
    lKeywords.append('inout')
    lKeywords.append('out')
    lKeywords.append('sla')
    lKeywords.append('constant')
    lKeywords.append('package')
    lKeywords.append('sll')
    lKeywords.append('disconnect')
    lKeywords.append('label')
    lKeywords.append('sra')

    sWord = remove_text_after_word(')', sWord)
    sWord = remove_text_after_word(';', sWord)

    if sWord.lower() in lKeywords:
        return True
    else:
        return False


def clear_keyword_from_line(oLine, sKeyword):
    '''
    Removes a keyword from a line.

    Parameters:

      oLine: (line object)

      sKeyword: (string)
    '''
    oLine.update_line(re.sub(r'^(\s*)' + sKeyword, r'\1  ', oLine.line))
    if re.match('^\s*$', oLine.line):
        oLine.update_line('')
        oLine.isBlank = True


def search_for_and_remove_keyword(oFile, iLineNumber, sKeyword):
    '''
    Searches for a keyword on lines below the current line and removes it if discovered.

    Parameters:

      oFile: (vhdlFile object)

      iLineNumber: (integer)

      sKeyword: (string)
    '''
    iSearchIndex = iLineNumber
    while True:
        iSearchIndex += 1
        oLine = oFile.lines[iSearchIndex]
        if re.match('^\s*' + sKeyword, oLine.line, re.IGNORECASE):
            clear_keyword_from_line(oLine, sKeyword)
            remove_blank_line(oFile, iSearchIndex)
        if not oLine.isBlank:
            break


def remove_comment(sString):
    '''
    Returns a string without comments.

    Parameters:

      sString: (string)

    Returns: (string)
    '''
    if '--' in sString:
        return sString[0:sString.find('--') + len('--')]
    else:
        return sString


def remove_blank_line(oFile, iLineNumber):
    '''
    Removes a line if it is blank.

    Parameters:

      oFile: (vhdlFile object)

      iLineNumber: (integer)
    '''
    if oFile.lines[iLineNumber].isBlank:
        oFile.lines.pop(iLineNumber)


def reclassify_line(oFile, iLineNumber):
    '''
    Updates the following attributes on the current and next line:

       * isFunctionReturn
       * insideVariableAssignment
       * isVariableAssignmentEnd
       * isVariableAssignment
       * insideSequential
       * isSequentialEnd
       * isSequential
       * hasComment
       * hasInlineComment
       * commentColumn

    Parameters:

      oFile: (vhdlFile object)

      iLineNumber: (integer)
    '''
    if re.match('^\s*return', oFile.lines[iLineNumber + 1].line, re.IGNORECASE):
        oFile.lines[iLineNumber].isFunctionReturn = False
        oFile.lines[iLineNumber + 1].isFunctionReturn = True
    elif re.match('^\s*\w+.*:=', oFile.lines[iLineNumber + 1].line, re.IGNORECASE):
        oFile.lines[iLineNumber + 1].insideVariableAssignment = True
        oFile.lines[iLineNumber + 1].isVariableAssignmentEnd = True
        oFile.lines[iLineNumber + 1].isVariableAssignment = True
    elif re.match('^\s*null', oFile.lines[iLineNumber + 1].line, re.IGNORECASE):
        oFile.lines[iLineNumber + 1].insideSequential = False
        oFile.lines[iLineNumber + 1].isSequentialEnd = False
        oFile.lines[iLineNumber + 1].isSequential = False
        oFile.lines[iLineNumber + 1].sequentialAlignmentColumn = None
    else:
        oFile.lines[iLineNumber + 1].insideSequential = True
        oFile.lines[iLineNumber + 1].isSequentialEnd = True
        oFile.lines[iLineNumber + 1].isSequential = True

    if '--' not in oFile.lines[iLineNumber].line:
        oFile.lines[iLineNumber].hasComment = False
        oFile.lines[iLineNumber].hasInlineComment = False
        oFile.lines[iLineNumber].commentColumn = None

    if '--' in oFile.lines[iLineNumber + 1].line:
        oFile.lines[iLineNumber + 1].hasComment = True
        oFile.lines[iLineNumber + 1].hasInlineComment = True
        oFile.lines[iLineNumber + 1].commentColumn = oFile.lines[iLineNumber + 1].line.find('--')


def remove_parenthesis_from_word(sWord):
    '''
    Removes parenthesis from words:

        Hello(there) => Hello
        Hello        => Hello

    Parameters:

      sWord: (string)

    Returns: (string)
    '''
    if '(' in sWord:
        return sWord.split('(')[0]

    return sWord


def strip_semicolon_from_word(sWord):
    '''
    Removes trailing semicolon from a word:

        case;   => case
        entity; => entity

    Parameters:

      sWord: (string)

    Returns: (string)
    '''
    if ';' == sWord[-1]:
        return sWord[0:-1]

    return sWord


def is_port_mode(sWord):
    '''
    Returns True if given word is a valid port mode.

    Returns False if given word is not a valid port mode.

    Parameters:

      sWord: (string)

    Returns: (boolean)
    '''
    lKeywords = []
    lKeywords.append('in')
    lKeywords.append('out')
    lKeywords.append('inout')
    lKeywords.append('buffer')
    lKeywords.append('linkage')

    if sWord.lower() in lKeywords:
        return True
    else:
        return False


def end_of_line_index(oLine):
    '''
    Finds the end of the code on a line ignoring comments.
    Returns the index of the last code character.

    Parameters:

      oLine: (line object)

    Returns: (integer)
    '''

    sLine = remove_comment(oLine.line).replace('--', '')
    for iIndex, sChar in enumerate(sLine[::-1]):
        if not sChar == ' ':
            return len(sLine) - iIndex


def remove_closing_parenthesis_and_semicolon(oLine):
    '''
    Parameters:

      oLine: (line object)

    Returns: (line object)
    '''
    return re.sub(r'\)(\s*);', r' \1 ', oLine)


def extract_non_keywords(sString):
    '''
    Returns a keyword list with the following removed:
       :'s
       commas
       semicolons
       vhdl keywords
       double quotes
       numbers
       ticks
       comments

    Parameters:

       sString: (string)

    Returns: (list of strings)
    '''
    lReturn = []
    sMyString = remove_comment(sString).replace('--', ' ')
    sMyString = sMyString.replace(':', ' ')
    sMyString = sMyString.replace(',', ' ')
    sMyString = sMyString.replace('\'', ' ')
    sMyString = sMyString.replace('(', ' ')
    sMyString = sMyString.replace(')', ' ')
    sMyString = sMyString.replace(';', ' ')
    sMyString = sMyString.replace('=', ' ')
    sMyString = sMyString.replace('>', ' ')
    sMyString = sMyString.replace('<', ' ')
    sMyString = sMyString.replace('&', ' ')
    sMyString = sMyString.replace('*', ' ')
    sMyString = sMyString.replace('-', ' ')
    sMyString = sMyString.replace('+', ' ')
    sMyString = sMyString.replace('/', ' ')
    sMyString = re.sub('x"\S+"', ' ', sMyString)
    sMyString = re.sub('X"\S+"', ' ', sMyString)

    for sWord in sMyString.split():
        if re.match('[0-9]+', sWord):
            continue
        if not is_vhdl_keyword(sWord):
            lReturn.append(sWord)

    return lReturn


def extract_class_name(oLine):
    '''
    Returns the name of a type in a type declaration.

    Parameters:

       oLine: (line object)

    Returns: (one element list of strings)
    '''
    return [oLine.line.split()[0]]


def extract_class_identifier_list(oLine):
    '''
    Returns a class identifiers list.

    Parameters:

       oLine: (line object)

    Returns: (list of strings)
    '''
    sLine = oLine.line.replace(';', '')
    sLine = sLine.split(':')[0]
    sLine = sLine.replace(',', ' ').split()
    if sLine[0].lower() in ['constant', 'variable', 'signal', 'file']:
        sLine = sLine[1:]

    return sLine


def _extract_type_name_from_type_declaration(sLine):
    '''
    Returns the name of a type in a type declaration.

    Parameters:

       sLine: (line)

    Returns: (one element list of strings)
    '''
    words = sLine.lower().split()
    return [sLine.split()[words.index('of') + 1]]


def extract_type_name(oLine):
    '''
    Returns the name of a type in various declarations.

    Parameters:

       oLine: (line object)

    Returns: (zero or one element list of strings)
    '''
    sLine = oLine.line.replace(';', '')
    sLine = sLine.replace('(', ' ')
    sLine = sLine.replace(')', ' ')

    if get_first_word(oLine).lower() == 'type':
        return _extract_type_name_from_type_declaration(sLine)
    else:
        try:
            sLine = sLine.split(':')[1]
            return [sLine.split()[0]]
        except:
            return []


def extract_type_name_vhdl_only(oLine):
    '''
    Returns the name of a VHDL only types in various declarations.

    Parameters:

       oLine: (line object)

    Returns: (one element or empty list of strings)
    '''
    name = extract_type_name(oLine)

    if name and is_vhdl_keyword(name[0]):
        return name

    return []


def extract_type_identifier(oLine):
    '''
    Returns the type identifier from type declaration.

    Parameters:

       oLine: (line object)

    Returns: (one element list of strings)
    '''
    return [oLine.line.split()[1]]


def extract_label(oLine):
    '''
    Returns the label.

    Parameters:

       oLine: (line object)

    Returns: (one element list of strings)
    '''
    return [oLine.line.replace(' ', '').split(':')[0]]


def extract_end_label(oLine):
    '''
    Returns the end label.

    Parameters:

       oLine: (line object)

    Returns: (one element or empty list of strings)
    '''
    word = oLine.line.replace(';', '').split()[-1]
    lower = word.lower()

    if lower == 'end' or lower == 'generate' or lower == 'process':
        return []

    return [word]


def extract_entity_identifier(oLine):
    '''
    Returns the entity identifier.

    Parameters:

       oLine: (line object)

    Returns: (one element list of strings)
    '''
    return [oLine.line.split()[1]]


extract_component_identifier = extract_entity_identifier


def extract_first_keyword(oLine):
    '''
    Returns first keyword from line.

    Parameters:

       oLine: (line object)

    Returns: (one element list of strings)
    '''
    sLine = oLine.line.replace('(', ' ')
    sLine = sLine.replace(':', ' ')

    for word in sLine.split():
        if is_vhdl_keyword(word):
            return [word]


def extract_port_name(oLine):
    '''
    Returns port name from line.

    Parameters:

       oLine: (line object)

    Returns: (one element list of strings)
    '''
    sLine = oLine.line.replace('(', ' ')
    return [sLine.split()[0]]


def extract_word(oLine, keyword):
    '''
    Returns is keyword from line.

    Parameters:

       sLine: (line object)

       keyword: keyword to extract

    Returns: (one element or empty list of strings)
    '''
    line = oLine.line.replace(';', '')
    line = line.replace('(', ' ')
    line = line.replace(')', ' ')

    for word in line.split():
        if word.lower() == keyword:
            return [word]

    return []


def extract_generics(oLine):
    '''
    Returns a generics list.

    Parameters:

       oLine: (line object)

    Returns: (list of strings)
    '''
    sLine = oLine.line.replace(';', '')
    sLine = sLine.split(':')[0]
    sLine = sLine.replace('(', ' ')
    sLine = sLine.replace(',', ' ').split()
    if sLine[0].lower() == 'generic':
        sLine = sLine[1:]

    return sLine


def remove_comment_attributes_from_line(oLine):
    '''
    Sets all comment attributes on a line to indicate no comment is present.

    Parameters:

        oLine: (line object)
    '''
    oLine.isComment = False
    oLine.hasComment = False
    oLine.hasInlineComment = False
    oLine.commentColumn = None


def is_number(sString):
    '''
    Returns boolean if the string passed is a number.

    Parameters:

        sLine: (string)

    Returns:  boolean
    '''
    sMyString = sString
    if sString.startswith('-'):
        sMyString = sString[1:]
    sMyString = sMyString.replace('.', '0')
    return sMyString.isdigit()


def remove_line(oFile, iLineNumber):
    '''
    Removes a line from the file line list.

    Parameters:

        oFile: (File Object)

        iLineNumber : (integer)

    Returns:  Nothing
    '''
    del oFile.lines[iLineNumber]


def remove_lines(oFile, iStartLine, iEndLine):
    '''
    Removes a series of lines from the file line list.

    Parameters:

        oFile: (File Object)

        iStartLine: (integer)

        iEndLine: (integer)

    Returns:  Nothing
    '''
    del oFile.lines[iStartLine:iEndLine + 1]


def insert_line(oFile, iIndex):
    '''
    Inserts a blank line at iIndex into the file line list.

    Parameters:

        oFile: (File Object)

        iIndex: (integer)

    Returns:  Nothing
    '''
    oFile.lines.insert(iIndex, line.blank_line())

def update_comment_line_attributes(oLine):
    '''
    Updates the following attributes on a line:

      self.isComment
      self.hasComment
      self.hasInLineComment
      self.commentColumn

    Parameters:

      oLine: (Line Object)

    Returns:  Nothing
    '''

    if re.match('^\s*--', oLine.line):
        oLine.isComment = True
        oLine.hasComment = True
        oLine.hasInlineComment = False
        oLine.commentColumn = oLine.line.find('--')
    elif '--' in oLine.line:
        oLine.isComment = False
        oLine.hasComment = True
        oLine.hasInlineComment = True
        oLine.commentColumn = oLine.line.find('--')
    else:
        oLine.isComment = False
        oLine.hasComment = False
        oLine.hasInlineComment = False
        oLine.commentColumn = None



'''
This module contains functions for rules to fix issues.
'''
from vsg import line
from vsg import utils

import re


def indent(self, oLine):
    '''
    Fixes indent violations.

    Parameters:

      self: (rule object)

      oLine: (line object)
    '''
    oLine.update_line(' '*oLine.indentLevel*self.indentSize + oLine.line.lstrip())


def keyword_alignment(self, oFile):
    '''
    Aligns keywords across multiple lines.

    Parameters:

      self: (rule object)

      oFile: (vhdlFile object)
    '''
    for sKey in self.dFix['violations']:
        iMaximumKeywordColumn = self.dFix['violations'][sKey]['maximumKeywordColumn']
        for iLineNumber in self.dFix['violations'][sKey]['line']:
            iKeywordColumn = self.dFix['violations'][sKey]['line'][iLineNumber]['keywordColumn']
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(oLine.line[:iKeywordColumn] + ' '*(iMaximumKeywordColumn - iKeywordColumn) + oLine.line[iKeywordColumn:])


def multiline_alignment(self, oFile, iLineNumber):
    '''
    Indents successive lines of multiline statements.

    Parameters:

      self: (rule object)

      oFile: (vhdlFile object)

      iLineNumber: (integer)
    '''
    oLine = oFile.lines[iLineNumber]
    oLine.update_line(' '*self.dFix['violations'][iLineNumber]['column'] + oLine.line.lstrip())


def lower_case(oLine, sKeyword):
    '''
    Changes word to lowercase.

    Parameters:

      self: (rule object)

      oLine: (line object)

      sKeyword: (string)
    '''
    utils.change_word(oLine, sKeyword, sKeyword.lower())


def upper_case(oLine, sKeyword):
    '''
    Changes word to lowercase.

    Parameters:

      self: (rule object)

      oLine: (line object)

      sKeyword: (string)
    '''
    if '(' in sKeyword:
        sWord = utils.remove_parenthesis_from_word(sKeyword)
        utils.change_word(oLine, sWord, sWord.upper())
    else:
        utils.change_word(oLine, sKeyword, sKeyword.upper())


def upper_case_with_parenthesis(self, oLine, sKeyword):
    '''
    Changes word to lowercase.

    Parameters:

      self: (rule object)

      oLine: (line object)

      sKeyword: (string)
    '''


def enforce_one_space_after_word(self, oLine, sWord):
    '''
    Adds a space after a word.

    Parameters:

      self: (rule object)

      oLine: (line object)

      sWord: (string)
    '''
    enforce_spaces_after_word(self, oLine, sWord, 1)


def enforce_spaces_after_word(self, oLine, sWord, iSpaces):
    '''
    Adds a space after a word.

    Parameters:

      self: (rule object)

      oLine: (line object)

      sWord: (string)

      iSpaces: (integer)
    '''
    oLine.update_line(re.sub(r'(' + sWord + ')(\s*)', r'\1' + ' ' * iSpaces, oLine.line, 1, flags=re.IGNORECASE))


def enforce_one_space_before_word(self, oLine, sWord, fWholeWord=False):
    '''
    Adds a space before word.

    Parameters:

      self: (rule object)

      oLine: (line object)

      sWord: (string)
    '''
    if fWholeWord:
        oLine.update_line(re.sub(r'(\S)\s+(' + sWord + ')', r'\1 \2', oLine.line, 1, flags=re.IGNORECASE))
    else:
        oLine.update_line(re.sub(r'(\S)\s*(' + sWord + ')', r'\1 \2', oLine.line, 1, flags=re.IGNORECASE))


def remove_blank_lines_above(self, oFile, iLineNumber, sUnless=None):
    '''
    This function removes blank lines above a linenumber.
    If sUnless is specified, a single blank line will be left if a line with the sUnless attribute is encountered.

    Parameters:

      self: (rule object)

      oFile: (vhdlFile object)

      iLineNumber: (integer)

      sUnless: (string) (optional)
    '''
    while oFile.lines[iLineNumber - 1].isBlank:
        if sUnless:
            if oFile.lines[iLineNumber - 2].__dict__[sUnless]:
                break
        oFile.lines.pop(iLineNumber - 1)
        iLineNumber -= 1


def remove_blank_lines_below(self, oFile, iLineNumber, sUnless=None):
    '''
    This function removes blank lines below a linenumber.
    If sUnless is specified, a single blank line will be left if a line with the sUnless attribute is encountered.

    Parameters:

      self: (rule object)

      oFile: (vhdlFile object)

      iLineNumber: (integer)

      sUnless: (string) (optional)
    '''
    while oFile.lines[iLineNumber + 1].isBlank:
        if sUnless:
            if oFile.lines[iLineNumber + 2].__dict__[sUnless]:
                break
        oFile.lines.pop(iLineNumber + 1)


def insert_blank_line_above(self, oFile, iLineNumber):
    '''
    This function inserts a blank line above the line specified by iLineNumber.

    Parameters:

      self: (rule object)

      oFile: (vhdlFile object)

      iLineNumber: (integer)
    '''
    oFile.lines.insert(iLineNumber, line.blank_line())


def insert_blank_line_below(self, oFile, iLineNumber):
    '''
    This function inserts a blank line below the line specified by iLineNumber.

    Parameters:

      self: (rule object)

      oFile: (vhdlFile object)

      iLineNumber: (integer)
    '''
    oFile.lines.insert(iLineNumber + 1, line.blank_line())


def replace_is_keyword(oFile, iLineNumber):
    '''
    This function removes the is keyword from a line if it starts with is.
    If the line is empty, it is replaced with a blank line.

    Parameters:

      oFile: (vhdlFile object)

      iLineNumber: (integer)
    '''
    iSearchIndex = iLineNumber
    while True:
        iSearchIndex += 1
        oLine = oFile.lines[iSearchIndex]
        if re.match('^\s*is', oLine.line, re.IGNORECASE):
            oLine.line = re.sub(r'^(\s*)is', r'\1  ', oLine.line)
            oLine.lineLower = oLine.line.lower()
            if re.match('^\s*$', oLine.line):
                oLine.line = ''
                oLine.lineLower = ''
                oLine.isBlank = True
        if oFile.lines[iSearchIndex].isGenericKeyword or oFile.lines[iSearchIndex].isPortKeyword:
            break


def identifier_alignment(self, oFile):
    '''
    Aligns identifiers and colons across multiple lines.

    Parameters:

      self: (rule object)

      oFile: (vhdlFile object)
    '''
    for sKey in self.dFix['violations']:
        for iLineNumber in self.dFix['violations'][sKey]['line']:
            oLine = oFile.lines[iLineNumber]
            lLine = oLine.line.split(':', 1)
            sKeyword = lLine[0].split()[0]
            sIdentifier = lLine[0].split()[1]
            sLine = ' '*oLine.indentLevel*self.indentSize
            sLine += sKeyword
            sLine += ' '*(self.dFix['violations'][sKey]['maximumKeywordLength'] - len(sKeyword) + 1)
            sLine += sIdentifier
            sLine += ' '*(self.dFix['violations'][sKey]['maximumIdentifierLength'] - len(sIdentifier))
            sLine += ' :' + lLine[1]
            oLine.update_line(sLine)




from vsg.rules import single_space_after_rule


class rule_002(single_space_after_rule):
    '''
    Function rule 002 checks there is a single space between the function keyword and the function name.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'function', '002', 'isFunctionKeyword', 'function')
        self.solution = 'Ensure a single space exists between the "function" keyword and the function name.'




from vsg import rule

import re


class rule_019(rule.rule):
    '''
    Case rule 019 checks for labels before the case case keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'case', '019')
        self.phase = 1
        self.solution = 'Remove label before "case" keyword'

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.hasCaseLabel:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub('^(\s*)(\w+\s*:\s*)', r'\1', oLine.line, 1))
            oLine.hasCaseLabel = False




from vsg import rule


class rule_001(rule.rule):
    '''
    Checks the length of the file and determines if the file is empty.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'source_file', '001')
        self.phase = 1
        # These are filled out when creating a new rule
        self.name = 'source_file'
        self.identifier = '001'
        self.fixable = False
        self.solution = 'File not found.'

    def analyze(self, oFile):
        if len(oFile.lines) == 1:
            self.add_violation(0)




from vsg.rules.concurrent import keyword_alignment_rule


class rule_008(keyword_alignment_rule):
    '''
    Concurrent rule 008 ensures the alignment of comments in sequential conccurent statements.
    '''

    def __init__(self):
        keyword_alignment_rule.__init__(self, 'concurrent', '008')
        self.solution = 'Inconsistent alignment of comments in group of lines.'
        self.sKeyword = '--'
        self.sStartGroupTrigger = 'isConcurrentBegin'
        self.sEndGroupTrigger = 'insideConcurrent'



'''
This module contains functions for rules to perform their checks.
'''

import re


def multiline_alignment(self, iColumn, oLine, iLineNumber):
    '''
    Checks the alignment of multiline statements.

    Parameters:

      self: (rule object)

      iColumn: (integer)

      oLine: (line object)

      iLineNumber: (integer)
    '''
    if not re.match('\s{' + str(iColumn) + '}\S', oLine.line):
        self.add_violation(iLineNumber)
        self.dFix['violations'][iLineNumber] = {}
        self.dFix['violations'][iLineNumber]['column'] = iColumn




from vsg import rule
from vsg import check
from vsg import fix


class rule_001(rule.rule):
    '''
    Instantiation rule 001 checks for proper indent of instantiations.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '001'
        self.solution = 'Ensure proper indentation.'
        self.phase = 4

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationDeclaration or oLine.isInstantiationPortAssignment or \
           oLine.isInstantiationPortEnd or oLine.isInstantiationPortKeyword or \
           oLine.isInstantiationGenericAssignment or oLine.isInstantiationGenericEnd or \
           oLine.isInstantiationGenericKeyword:
            check.indent(self, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.indent(self, oFile.lines[iLineNumber])




from vsg.rules import single_space_after_rule


class rule_006(single_space_after_rule):
    '''
    Case rule 006 checks for a single space between the "end" and "case" keywords.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'case', '006', 'isEndCaseKeyword', 'end')
        self.solution = 'Ensure a single space exists between the "end" and "case" keywords.'




from vsg import rule
from vsg import check
from vsg import fix


class rule_003(rule.rule):
    '''
    Port rule 003 checks spacing between "port" and the open parenthesis.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'port'
        self.identifier = '003'
        self.solution = 'Change spacing between "port" and "(" to one space.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPortKeyword and '(' in oLine.line:
            check.is_single_space_after(self, 'port', oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            fix.enforce_one_space_after_word(self, oLine, 'port')




from vsg import rule
from vsg import fix

import re


class rule_013(rule.rule):
    '''
    Process rule 013 checks the "is" keyword is lowercase.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '013')
        self.solution = 'Lowercase "is" keyword.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isSensitivityListEnd and re.match('^.*\)\s*is', oLine.lineLower):
            if not re.match('^.*\)\s*is', oLine.line):
                    self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(oFile.lines[iLineNumber], 'is')




from vsg.rules import prefix_rule
from vsg import utils


class rule_015(prefix_rule):
    '''
    Constant rule 015 checks for prefixes in constant identifiers.
    '''

    def __init__(self):
        prefix_rule.__init__(self, 'constant', '015', 'isConstant')
        self.prefixes = ['c_']
        self.solution = 'Constant'

    def _extract(self, oLine):
        return utils.extract_class_identifier_list(oLine)




import os
import importlib
import inspect

from . import junit


def get_python_modules_from_directory(sDirectoryName, lModules):
    '''
    Returns a list of files with an extension of py from a directory.
    It ignores files starting with a double underscore __.

    Parameters:

      sDirectoryName (string)

    Modifies:

      lModules (string list)
    '''
    try:
        lDirectoryContents = os.listdir(sDirectoryName)
        for sFileName in lDirectoryContents:
            if sFileName.endswith('.py') and not sFileName.startswith('__'):
                lModules.append(sFileName.replace('.py', ''))
    except OSError:
        print('ERROR: specified local rules directory ' + sDirectoryName + ' could not be found.')
        exit()


def get_rules_from_module(lModules, lRules):
    '''
    Returns a list of files that start with "rule_".

    Parameters:

      lModules (list)

    Modifies:

      lRules (object list)
    '''
    for sModuleName in lModules:
        for name, obj in inspect.getmembers(importlib.import_module(sModuleName)):
            if name.startswith('rule_'):
                lRules.append(obj())


def load_local_rules(sDirectoryName):
    '''
    Loads rules from the directory passed to this routine.

    Parameters:

      sDirectoryName (string)

    Returns: (string list)
    '''
    lLocalModules = []
    get_python_modules_from_directory(sDirectoryName, lLocalModules)

    lRules = []
    get_rules_from_module(lLocalModules, lRules)
    return lRules


def load_rules():
    '''
    Loads rules from the vsg/rules directory.

    Parameters:  None

    Returns:  (rule object list)
    '''
    lRules = []
    for name, oPackage in inspect.getmembers(importlib.import_module('vsg.rules')):
        if inspect.ismodule(oPackage):
            for name, oRule in inspect.getmembers(oPackage):
                if inspect.isclass(oRule) and name.startswith('rule_'):
                    lRules.append(oRule())

    return lRules


def maximum_phase(lRules):
    '''
    Determines the maximum phase number from all the rules.

    Parameters:
      lRules (rule object list)

    Returns: (integer)
    '''
    maximumPhaseNumber = 0
    for oRule in lRules:
        if oRule.phase > maximumPhaseNumber:
            maximumPhaseNumber = oRule.phase
    return maximumPhaseNumber


class rule_list():
    '''
    Contains a list of all rules to be checked.
    It loads all base rules.
    Localized rules are loaded if specified.

    Parameters:

      oVhdlFile: (vhdlFile object)

      sLocalRulesDirectory: (string) (optional)
    '''
    def __init__(self, oVhdlFile, sLocalRulesDirectory=None):
        self.rules = (load_rules())
        if sLocalRulesDirectory:
            self.rules.extend(load_local_rules(sLocalRulesDirectory))
        self.iNumberRulesRan = 0
        self.lastPhaseRan = 0
        self.oVhdlFile = oVhdlFile
        self.maximumPhase = maximum_phase(self.rules)
        self.violations = False

    def fix(self, iPhase):
        '''
        Applies fixes to all violations found.

        Parameters:

          iPhase: (integer)
        '''
        for phase in range(1, int(iPhase) + 1):
            for oRule in self.rules:
                if oRule.phase == phase and not oRule.disable:
                    oRule.fix(self.oVhdlFile)

    def check_rules(self):
        '''
        Analyzes all rules in increasing phase order.
        If there is a violation in a phase, analysis is halted.

        Parameters:  None
        '''
        self.iNumberRulesRan = 0
        iFailures = 0
        self.violations = False
        for phase in range(1, 10):
            for oRule in self.rules:
                if oRule.phase == phase and not oRule.disable:
                    oRule.analyze(self.oVhdlFile)
                    iFailures += len(oRule.violations)
                    self.iNumberRulesRan += 1
                    self.lastPhaseRan = phase
            if iFailures > 0:
                self.violations = True
                break

    def report_violations(self, sOutputFormat):
        '''
        Prints out violations to stdout.

        Parameters:

          sOutputFormat (string)
        '''
        if sOutputFormat == 'vsg':
            sFileTitle = 'File:  ' + self.oVhdlFile.filename
            print(sFileTitle)
            print('=' * len(sFileTitle))
        iFailures = 0
        for phase in range(1, self.maximumPhase + 1):
            if phase <= self.lastPhaseRan:
                if sOutputFormat == 'vsg':
                    print('Phase ' + str(phase) + '... Reporting')
                for iLineNumber in range(0, len(self.oVhdlFile.lines)):
                    for oRule in self.rules:
                        if oRule.phase == phase:
                            iFailures += oRule.report_violations(iLineNumber, sOutputFormat, self.oVhdlFile.filename)
            else:
                if sOutputFormat == 'vsg':
                    print('Phase ' + str(phase) + '... Not executed')

        if sOutputFormat == 'vsg':
            print('=' * len(sFileTitle))
            print('Total Rules Checked: ' + str(self.iNumberRulesRan))
            print('Total Violations:    ' + str(iFailures))

    def configure(self, configurationFile):
        '''
        Configures individual rules based on dictionary passed.

        Parameters:

          configurationFile: (dictionary)
        '''
        if configurationFile and 'rule' in configurationFile:
            self._validate_configuration_rule_exists(configurationFile)
            for oRule in self.rules:
                oRule.configure(configurationFile)

    def _validate_configuration_rule_exists(self, configurationFile):
        '''
        Validates rules called out in the configuration files exist in the rule set.

        If a rule does not exist then:

          1) an error message will be printed
          2) tool will exit with a status of 1

        Parameters:

          configurationFile: (dictionary)

        Returns:  nothing
        '''
        lRuleNames = []
        for oRule in self.rules:
            lRuleNames.append(oRule.name + '_' + oRule.identifier)
        for sRule in configurationFile['rule']:
            if not sRule == 'global' and sRule not in lRuleNames:
                print('ERROR: Rule ' + sRule + ' referenced in configuration could not be found')
                exit(1)

    def extract_junit_testcase(self, sVhdlFileName):
        '''
        Creates JUnit XML file listing all violations found.

        Parameters:

          sVhdlFileName (string)

        Returns: (junit testcase object)
        '''
        oTestcase = junit.testcase(sVhdlFileName, str(0), 'failure')
        oFailure = junit.failure('Failure')
        for oRule in self.rules:
            if len(oRule.violations) > 0:
                for iLinenumber in oRule.violations:
                    oFailure.add_text(oRule.name + '_' + oRule.identifier + ': ' + str(iLinenumber) + ' : ' + oRule.solution)
                oTestcase.add_failure(oFailure)

        return oTestcase

    def get_configuration(self):
        '''
        Returns a dictionary with every rule and how it is configured.

        Parameters:

          None

        Returns: (dictionary)
        '''
        dConfiguration = {}
        for oRule in self.rules:
            sId = oRule.name + '_' + oRule.identifier
            dConfiguration[sId] = oRule.get_configuration()
        return dConfiguration




from vsg.rules import single_space_after_character_rule


class rule_006(single_space_after_character_rule):
    '''
    Generic rule 006 checks for a single space after the default assignment in a generic declaration.
    '''

    def __init__(self):
        single_space_after_character_rule.__init__(self, 'generic', '006', 'isGenericDeclaration', ':=')
        self.solution = 'Reduce number of spaces after the default assignment to 1.'



import re


def comment(dVars, oLine):
    # Check for comment lines
    if '--' in oLine.line:
        oLine.hasComment = True
        oLine.commentColumn = oLine.line.find('--')
        if re.match('^\s*--', oLine.line):
            oLine.isComment = True
            oLine.indentLevel = dVars['iCurrentIndentLevel']
        else:
            oLine.hasInlineComment = True




from vsg import rule
from vsg import fix
from vsg import check

import re


class rule_012(rule.rule):
    '''
    Entity rule 012 checks entity name is uppercase in "end" keyword line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'entity'
        self.identifier = '012'
        self.solution = 'Uppercase entity name.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEndEntityDeclaration and re.match('^\s*end\s+entity\s+\w+', oLine.line, re.IGNORECASE):
            lLine = oLine.line.split()
            check.is_uppercase(self, lLine[2], iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.upper_case(oFile.lines[iLineNumber], oFile.lines[iLineNumber].line.split()[2])




from vsg.rules import indent_rule


class rule_003(indent_rule):
    '''
    Process rule 003 checks for the proper indentation at the beginning of the line.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'process', '003', 'isProcessBegin')




from vsg import rule
from vsg import check
from vsg import fix
from vsg import utils


class rule_018(rule.rule):
    '''
    Port rule 018 checks the port type is lowercase.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'port', '018')
        self.solution = 'Change port type to lowercase.'
        self.phase = 6

    def analyze(self, oFile):
        for iLineNumber, oLine in enumerate(oFile.lines):
            if self._is_vsg_off(oLine):
                continue
            if oLine.isPortDeclaration:
                sLine = oLine.line.split(':')[1]
                if '(' in sLine:
                    sLine = sLine.split('(')[0]
                if not utils.is_port_mode(sLine.split()[0]):
                    continue
                if utils.is_vhdl_keyword(sLine.split()[1]):
                    check.is_lowercase(self, sLine.split()[1], iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            sLine = oLine.line.split(':')[1]
            if '(' in sLine:
                sLine = sLine.split('(')[0]
            sWord = sLine.split()[1]
            fix.lower_case(oLine, sWord)




from vsg.rules import single_space_after_character_rule


class rule_005(single_space_after_character_rule):
    '''Generic rule 005 checks for a single space after the colon in a generic declaration.'''

    def __init__(self):
        single_space_after_character_rule.__init__(self, 'generic', '005', 'isGenericDeclaration', ':')
        self.solution = 'Reduce number of spaces after the colon to 1.'




def get_package_name(oLine):
    '''
    Returns the package name in the line.
    Parameters
      oLine: (line object)
    Returns: (string)
      Package name or empty string if package name not found.
    '''
    lLine = oLine.lineNoComment.split()
    for iIndex, sWord in enumerate(lLine):
        if sWord.lower() == 'package' and not lLine[iIndex + 1] == '--':
            return lLine[iIndex + 1].rstrip(';')
    return ''




from vsg import rule
from vsg import utils

import re


class rule_022(rule.rule):
    '''If rule 022 checks for code after the "else" keyword.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'if'
        self.identifier = '022'
        self.solution = 'Move code after "else" keyword to the next line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isElseKeyword and re.match('^.*else\s+\w', oLine.lineLower):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            utils.split_line_after_word(oFile, iLineNumber, 'else')
            oFile.lines[iLineNumber + 1].isElseKeyword = False
            oFile.lines[iLineNumber].isIfKeyword = False
            oFile.lines[iLineNumber].isElseIfKeyword = False
            oFile.lines[iLineNumber].isThenKeyword = False
            oFile.lines[iLineNumber + 1].indentLevel += 1
            oFile.lines[iLineNumber].isLastEndIf = False
            utils.reclassify_line(oFile, iLineNumber)
            oFile.lines[iLineNumber + 1].isFirstIf = False




from vsg import rule
from vsg import line
from vsg import utils

import re


class rule_010(rule.rule):
    '''Generic rule 010 checks the closing parenthesis for generics are on a line by itself.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'generic'
        self.identifier = '010'
        self.solution = 'Closing parenthesis must be on a line by itself.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEndGenericMap and not re.match('^\s*\)', oLine.line):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            oFile.lines[iLineNumber].line = utils.remove_closing_parenthesis_and_semicolon(oFile.lines[iLineNumber].line)
            oFile.lines[iLineNumber].isEndGenericMap = False
            oFile.lines.insert(iLineNumber + 1, line.line('  );'))
            oFile.lines[iLineNumber + 1].isEndGenericMap = True
            oFile.lines[iLineNumber + 1].insideGenericMap = True
            if oFile.lines[iLineNumber].isGenericKeyword:
                oFile.lines[iLineNumber + 1].indentLevel = oFile.lines[iLineNumber].indentLevel
            else:
                oFile.lines[iLineNumber + 1].indentLevel = oFile.lines[iLineNumber].indentLevel
                oFile.lines[iLineNumber].indentLevel = oFile.lines[iLineNumber].indentLevel + 1




from vsg import rule
from vsg import utils


class rule_009(rule.rule):
    '''
    Function rule 009 checks for a function parameter on the same line as the **function** keyword without a **begin** keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'function', '009')
        self.solution = 'Move function parameter to it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isFunctionParameter and oLine.isFunctionKeyword and 'return' not in oLine.line:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            utils.copy_line(oFile, iLineNumber)
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(oLine.line.split('(')[0] + '(')
            oLine.isFunctionParameter = False
            oLine = oFile.lines[iLineNumber + 1]
            oLine.update_line('  ' + oLine.line.split('(')[1])
            oLine.isFunctionKeyword = False
            oLine.indentLevel = oFile.lines[iLineNumber].indentLevel + 1




from vsg import rule
from vsg import check

import re


class architecture_case_rule(rule.rule):
    '''
    Architecture rule 013 checks the architecture name is upper case in the architecture declaration.
    '''

    def __init__(self, name=None, identifier=None, iIndex=None):
        rule.rule.__init__(self, name, identifier)
        self.phase = 6
        self.iIndex = iIndex

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isArchitectureKeyword and re.match('^\s*\S+\s\s*\S+\s\s*of\s\s*\S+\s\s*is', oLine.lineLower):
            lLine = oLine.line.split()
            check.is_uppercase(self, lLine[self.iIndex], iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            sLine = oLine.lineNoComment
            sWord = sLine.split()[self.iIndex]
            iIndex = sLine.rfind(sLine.split()[self.iIndex])
            sLine = sLine[:iIndex] + sWord.upper() + sLine[iIndex + len(sWord):]
            oLine.update_line(sLine)




from vsg.rules import case_rule
from vsg import utils


class rule_009(case_rule):
    '''
    Generic rule 009 checks the "generic" keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'generic', '009', 'isGenericKeyword')
        self.solution = 'Change "generic" keyword to '

    def _extract(self, oLine):
        return utils.extract_first_keyword(oLine)




from vsg import rule
from vsg import fix
from vsg import check


class rule_012(rule.rule):
    '''Component rule 012 checks component name is uppercase in "end" keyword line.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'component'
        self.identifier = '012'
        self.solution = 'Uppercase component name.'
        self.phase = 6

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isComponentEnd:
            lLine = oLine.line.split()
            if len(lLine) > 2:
                check.is_uppercase(self, lLine[2], iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            lLine = oFile.lines[iLineNumber].line.split()
            fix.upper_case(oFile.lines[iLineNumber], lLine[2])




from vsg import rule
from vsg import fix

import re


class rule_005(rule.rule):
    '''Component rule 005 checks the "is" keyword is on the same line as the component keyword.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'component'
        self.identifier = '005'
        self.solution = 'Add "is" keyword to same line as "component" keyword.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isComponentDeclaration and not re.match('^\s*component\s+\w+\s+is', oLine.line, re.IGNORECASE):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'^(\s*component\s+\w+)', r'\1 is', oLine.lineLower, re.IGNORECASE))
            # Search for "is" on the next line
            fix.replace_is_keyword(oFile, iLineNumber)




from vsg.rules import line_above_rule


class rule_009(line_above_rule):
    '''
    Case rule 009 ensures a blank line exists above the "end case" keywords.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'case', '009', 'isEndCaseKeyword')




from vsg.rules import single_space_after_character_rule


class rule_002(single_space_after_character_rule):
    '''
    Sequential rule 002 checks for a single space after the "<=" keyword.
    '''

    def __init__(self):
        single_space_after_character_rule.__init__(self, 'sequential', '002', 'isSequential', '<=')
        self.solution = 'Ensure a single space exists after the "<=" keyword.'




from vsg.rules import indent_rule


class rule_012(indent_rule):
    '''
    Constant rule 012 checks for the proper indentation of multiline constants.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'constant', '012', 'isConstantArray', 'isConstant')




from vsg.rules import single_space_after_character_rule


class rule_002(single_space_after_character_rule):
    '''
    Concurrent rule 002 checks there is a single space after the assignment.
    '''

    def __init__(self):
        single_space_after_character_rule.__init__(self, 'concurrent', '002', 'isConcurrentBegin', '<=')
        self.solution = 'Remove all but one space after the <=.'



#!/usr/bin/env python

import argparse
import sys
import os
import json
import shutil
import glob
import yaml

from . import rule_list
from . import vhdlFile
from . import junit
from . import version


def parse_command_line_arguments():
    '''Parses the command line arguments and returns them.'''

    parser = argparse.ArgumentParser(
      prog='VHDL Style Guide (VSG)',
      description='''Analyzes VHDL files for style guide violations.
                   Reference documentation is located at:
                   http://vhdl-style-guide.readthedocs.io/en/latest/index.html''')

    parser.add_argument('-f', '--filename', nargs='+', help='File to analyze')
    parser.add_argument('-lr', '--local_rules', help='Path to local rules')
    parser.add_argument('-c', '--configuration', nargs='+', help='JSON or YAML configuration file(s)')
    parser.add_argument('--fix', default=False, action='store_true', help='Fix issues found')
    parser.add_argument('-fp', '--fix_phase', default=10, action='store', help='Fix issues up to and including this phase')
    parser.add_argument('-j', '--junit', action='store', help='Extract Junit file')
    parser.add_argument('-of', '--output_format', action='store', default='vsg', choices=['vsg', 'syntastic'], help='Sets the output format.')
    parser.add_argument('-b', '--backup', default=False, action='store_true', help='Creates copy of input file for comparison with fixed version.')
    parser.add_argument('-oc', '--output_configuration', default=None, action='store', help='Output configuration file name')
    parser.add_argument('-v', '--version', default=False, action='store_true', help='Displays version information')

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)
    else:
        return parser.parse_args()


def open_configuration_file(sFileName, commandLineArguments):
    '''Attempts to open a configuration file and read it's contents.'''
    try:
        with open(sFileName) as yaml_file:
            tempConfiguration = yaml.full_load(yaml_file)
    except IOError:
        print('ERROR: Could not find configuration file: sFileName')
        write_invalid_configuration_junit_file(sFileName, commandLineArguments)
        exit()
    except yaml.scanner.ScannerError as e:
        print('ERROR: Invalid configuration file: ' + sFileName)
        print(e)
        write_invalid_configuration_junit_file(sFileName, commandLineArguments)
        exit()
    except yaml.parser.ParserError as e:
        print('ERROR: Invalid configuration file: ' + sFileName)
        print(e)
        write_invalid_configuration_junit_file(sFileName, commandLineArguments)
        exit()
    return tempConfiguration


def read_configuration_files(commandLineArguments):
    if commandLineArguments.configuration:
        dConfiguration = {}
        for sFileName in commandLineArguments.configuration:

            tempConfiguration = open_configuration_file(sFileName, commandLineArguments)

            for sKey in tempConfiguration.keys():
                if sKey == 'file_list':
                    try:
                        dConfiguration['file_list'].extend(tempConfiguration['file_list'])
                    except:
                        dConfiguration['file_list'] = tempConfiguration['file_list']
                elif sKey == 'rule':
                    for sRule in tempConfiguration[sKey]:
                        try:
                            dConfiguration[sKey][sRule] = tempConfiguration[sKey][sRule]
                        except:
                            dConfiguration[sKey] = {}
                            dConfiguration[sKey][sRule] = tempConfiguration[sKey][sRule]
                else:
                    dConfiguration[sKey] = tempConfiguration[sKey]

        return dConfiguration


def write_invalid_configuration_junit_file(sFileName, commandLineArguments):
    if commandLineArguments.junit:
        oJunitFile = junit.xmlfile(commandLineArguments.junit)
        oJunitTestsuite = junit.testsuite('vhdl-style-guide', str(0))
        oJunitTestcase = junit.testcase(sFileName, str(0), 'failure')
        oFailure = junit.failure('Failure')
        oFailure.add_text('Invalid JSON format.  Review configuration for errors.')
        oJunitTestcase.add_failure(oFailure)
        oJunitTestsuite.add_testcase(oJunitTestcase)
        oJunitFile.add_testsuite(oJunitTestsuite)
        write_junit_xml_file(oJunitFile)


def write_vhdl_file(oVhdlFile):
    with open(oVhdlFile.filename, 'w') as oFile:
        for oLine in oVhdlFile.lines[1:]:
            oFile.write(oLine.line + '\n')
    oFile.close()


def write_junit_xml_file(oJunitFile):
    with open(oJunitFile.filename, 'w') as oFile:
        for sLine in oJunitFile.build_junit():
            oFile.write(sLine + '\n')
    oFile.close()


def update_command_line_arguments(commandLineArguments, configuration):
    if configuration:
        if 'file_list' in configuration:
            for sFilename in configuration['file_list']:
                try:
                    commandLineArguments.filename.extend(glob.glob(expand_filename(sFilename)))
                except:
                    commandLineArguments.filename = glob.glob(expand_filename(sFilename))
        if 'local_rules' in configuration:
            commandLineArguments.local_rules = expand_filename(configuration['local_rules'])


def expand_filename(sFileName):
    '''Expands environment variables in filenames.'''
    return os.path.expanduser(os.path.expandvars(sFileName))


def create_backup_file(sFileName):
    '''Copies existing file and adds .bak to the end.'''
    shutil.copy2(sFileName, sFileName + '.bak')


def read_vhdlfile(sFileName):
    try:
        lLines = []
        with open(sFileName) as oFile:
            for sLine in oFile:
                lLines.append(sLine)
        oFile.close()
        return lLines
    except IOError:
        return []


def main():
    '''Main routine of the VHDL Style Guide (VSG) program.'''

    commandLineArguments = parse_command_line_arguments()

    version.print_version(commandLineArguments)

    configuration = read_configuration_files(commandLineArguments)

    update_command_line_arguments(commandLineArguments, configuration)

    # Add local rule path to system path so the rules can be loaded
    if commandLineArguments.local_rules:
        sys.path.append(os.path.abspath(commandLineArguments.local_rules))
    if commandLineArguments.junit:
        oJunitFile = junit.xmlfile(commandLineArguments.junit)
        oJunitTestsuite = junit.testsuite('vhdl-style-guide', str(0))
    fExitStatus = 0
    for sFileName in commandLineArguments.filename:
        oVhdlFile = vhdlFile.vhdlFile(read_vhdlfile(sFileName))
        oVhdlFile.filename = sFileName
        oRules = rule_list.rule_list(oVhdlFile, commandLineArguments.local_rules)
        oRules.configure(configuration)

        if commandLineArguments.fix:
            if commandLineArguments.backup:
                create_backup_file(sFileName)
            oRules.fix(commandLineArguments.fix_phase)
            write_vhdl_file(oVhdlFile)

        oRules.check_rules()
        oRules.report_violations(commandLineArguments.output_format)
        fExitStatus = update_exit_status(fExitStatus, oRules)

        if commandLineArguments.junit:
            oJunitTestsuite.add_testcase(oRules.extract_junit_testcase(sFileName))

    if commandLineArguments.junit:
        oJunitFile.add_testsuite(oJunitTestsuite)
        write_junit_xml_file(oJunitFile)

    if commandLineArguments.output_configuration:
        dOutputConfiguration = {}
        dOutputConfiguration['cwd'] = os.getcwd()
        dOutputConfiguration['file_list'] = []
        for sFileName in commandLineArguments.filename:
            dOutputConfiguration['file_list'].append(sFileName)
        if commandLineArguments.local_rules:
            dOutputConfiguration['local_rules'] = commandLineArguments.local_rules
        dOutputConfiguration['rule'] = oRules.get_configuration()
        with open(commandLineArguments.output_configuration, 'w') as json_file:
            json.dump(dOutputConfiguration, json_file, sort_keys=True, indent=2)

    sys.exit(0)


def update_exit_status(fExitStatus, oRules):
    if fExitStatus == 0 and oRules.violations:
        return 1
    else:
        return 0

if __name__ == '__main__':
    main()



import re


def package(dVars, oLine):

    # Check package declarations
    if re.match('^\s*package', oLine.lineLower) and not oLine.insidePackage and not oLine.insidePackageBody:
        oLine.isPackageKeyword = True
        oLine.insidePackage = True
        oLine.indentLevel = 0
        dVars['iCurrentIndentLevel'] = 1
    if oLine.insidePackage:
        if not oLine.insideProcess and not oLine.insideCase and not oLine.insideComponent and not oLine.insideTypeRecord:
            if re.match('^\s*end\s+', oLine.lineLower):
                oLine.isPackageEnd = True
                oLine.indentLevel = 0
                dVars['iCurrentIndentLevel'] = 0




from vsg.rules import indent_rule


class rule_007(indent_rule):
    '''
    Architecture rule 007 checks for spaces at the beginning of the line for the "begin" keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'architecture', '007', 'isArchitectureBegin')




from vsg import rule
from vsg import fix
from vsg import check
from abc import abstractmethod


class case_rule(rule.rule):
    '''
    Checks for and fixes words case.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    sTrigger : string
       The line attribute the rule applies to.

    Attribute
    ----------

    self.phase : integer = 6
       Sets the phase the rule will run in.

    self.solution : string = None
       Instructions on how to fix the violation.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None):
        rule.rule.__init__(self, name, identifier)
        self.phase = 6
        self.solution = None
        self.sTrigger = sTrigger
        self.case = 'lower'
        self.words_to_fix = set()

    @abstractmethod
    def _extract(self, oLine):
        pass

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.sTrigger]:
            words = self._extract(oLine)

            # ToDo: Raise exception when case differs from lower and upper
            if self.case == 'lower':
                check_function = check.is_lowercase
            else:
                check_function = check.is_uppercase

            for word in words:
                if check_function(self, word, iLineNumber) == False:
                    self.words_to_fix.add(word)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            for word in self.words_to_fix:
                if self.case == 'lower':
                    fix_function = fix.lower_case
                else:
                    fix_function = fix.upper_case

                fix_function(oFile.lines[iLineNumber], word)

    def _get_solution(self, iLineNumber):
       return self.solution + self.case + 'case.'




from vsg import rule

import re


class rule_012(rule.rule):
    '''
    Signal rule 012 checks the second signal in a signal declaration are aligned within the architecture declarative region.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'signal', '012')
        self.phase = 5
        self.solution = 'Align second signal with others.'

    def analyze(self, oFile):
        iMaxSignalIndex = 0
        lIndexes = []
        for iLineNumber, oLine in enumerate(oFile.lines):
            if self._is_vsg_off(oLine):
                continue
            if oLine.isSignal and re.match('^\s*signal\s+\S+,\s*\S+\s*:', oLine.line, flags=re.IGNORECASE):
                self.dFix[iLineNumber] = {}
                self.dFix[iLineNumber]['comma'] = 0
                for iIndex, sChar in enumerate(oLine.line):
                    if self.dFix[iLineNumber]['comma'] > 0 and not sChar == ' ':
                        iMaxSignalIndex = max(iMaxSignalIndex, iIndex + 1)
                        self.dFix[iLineNumber]['signal'] = iIndex + 1
                        break
                    if sChar == ',':
                        self.dFix[iLineNumber]['comma'] = iIndex + 1
                        lIndexes.append(iLineNumber)
            if oLine.isArchitectureBegin:
                for iIndex in lIndexes:
                    self.dFix[iIndex]['max'] = iMaxSignalIndex
                    if iMaxSignalIndex > self.dFix[iIndex]['signal']:
                        self.add_violation(iIndex)
                lIndexes = []

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            iComma = self.dFix[iLineNumber]['comma']
            iAddNumSpaces = self.dFix[iLineNumber]['max'] - self.dFix[iLineNumber]['signal']
            oLine.update_line(oLine.line[:iComma] + iAddNumSpaces*' ' + oLine.line[iComma:])




from vsg import rule


class rule_025(rule.rule):
    '''
    Architecture rule 025 checks for valid architecture names.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'architecture', '025')
        self.names = []
        self.solution = ''
        self.phase = 7
        self.fixable = False
        self.disable = True
        self.configuration.append('names')

    def _pre_analyze(self):
        self.lower_names = []
        for sName in self.names:
            self.lower_names.append(sName.lower())

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isArchitectureKeyword:
            lLine = oLine.line.split()
            if lLine[1].lower() not in self.lower_names:
                self.add_violation(iLineNumber)

    def _get_solution(self, iLineNumber):
        return 'Architecture name must be from this list: ' + ','.join(self.lower_names)




from vsg.rules import prefix_rule
from vsg import utils


class rule_008(prefix_rule):
    '''
    Signal rule 008 checks for prefixes in signal identifiers.
    '''

    def __init__(self):
        prefix_rule.__init__(self, 'signal', '008', 'isSignal')
        self.prefixes = ['s_']
        self.solution = 'Signal'

    def _extract(self, oLine):
        return utils.extract_class_identifier_list(oLine)
